/*
 * mem.c
 *
 * Copyright (c) 1995, John D. Kilburg <john@cs.unlv.edu>
 * See the file COPYRIGHT for details.
 */
#include <stdio.h>

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include "common.h"

#define ALLOC_AHEAD 3 /* must be >= 1 */

static long existing = 0;

/*
 * This should be a multiple of the word size.
 */
struct _pool
{
  size_t used, len;
  struct _pool *next;
};

struct _MemPool
{
  struct _pool *plist;
};

/*
 * MemPoolCreate
 */
MemPool
MemPoolCreate()
{
  struct _MemPool *mp;
  
  mp = (struct _MemPool *)alloc_mem(sizeof(struct _MemPool));
  mp->plist = NULL;

  existing++;

  return(mp);
}

/*
 * MemGet
 */
void *
MemPoolGet(mp, len)
MemPool mp;
size_t len;
{
  struct _pool *pool;
  byte *mem;
  size_t blen;

  len = (len / sizeof(word) + 1) * sizeof(word);

  pool = mp->plist;
  if (pool == NULL || (pool->used + len) > pool->len)
  {
    blen = ((len + sizeof(struct _pool)) / BUFSIZ + ALLOC_AHEAD) * BUFSIZ;
    pool = (struct _pool *)alloc_mem(blen);
    pool->len = blen;
    pool->used = sizeof(struct _pool);
    pool->next = mp->plist;
    mp->plist = pool;
  }

  mem = (byte *)pool + pool->used;
  pool->used += len;

  return(mem);
}

/*
 * MemPoolClearGet
 */
void *
MemPoolClearGet(mp, len)
MemPool mp;
size_t len;
{
  void *m;
  m = MemPoolGet(mp, len);
  memset(m, 0, len);
  return(m);
}

/*
 * MemPoolStrDup
 */
char *
MemPoolStrDup(mp, s)
MemPool mp;
char *s;
{
  size_t len;
  char *ns;

  if (s == NULL) return(NULL);
  len = strlen(s);
  ns = (char *)MemPoolGet(mp, len + 1);
  strcpy(ns, s);
  return(ns);
}

/*
 * MemPoolDestroy
 */
void
MemPoolDestroy(mp)
MemPool mp;
{
  struct _pool *m, *t;

  for (m = mp->plist; m != NULL; )
  {
    t = m;
    m = m->next;
    free_mem(t);
  }

  free_mem(mp);

  existing--;

  return;
}

/*
 * MemPoolExisting
 */
void
MemPoolExisting()
{
  fprintf (stderr, "EXISTING POOLS: %ld\n", existing);
  return;
}
