/*
 * font.c
 *
 * Copyright (c) 1995, John D. Kilburg <john@cs.unlv.edu>
 * See the file COPYRIGHT for details.
 */
#include <stdio.h>

#include "layout.h"

/*
 *
 * You really don't want to look in here.
 *
 */

#define XLFD_FOUNDRY 0
#define XLFD_FAMILY 1
#define XLFD_WEIGHT_NAME 2
#define XLFD_SLANT 3
#define XLFD_SETWIDTH_NAME 4
#define XLFD_STYLE_NAME 5
#define XLFD_PIXEL_SIZE 6
#define XLFD_POINT_SIZE 7
#define XLFD_RES_X 8
#define XLFD_RES_Y 9
#define XLFD_SPACING 9
#define XLFD_AVERAGE_WIDTH 10
#define XLFD_REGISTRY 11
#define XLFD_ENCODING 12
#define XLFD_COUNT 13

static void ParseXLFD _ArgProto((MemPool, LayoutClass *, char *,
				 LayoutFontInfo *));

/*
 * ParseXLFD
 *
 * Yipe!
 */
static void
ParseXLFD(mp, lc, xlfd, lfi)
MemPool mp;
LayoutClass *lc;
char *xlfd;
LayoutFontInfo *lfi;
{
  char *fields[XLFD_COUNT];
  int i;
  char *t;

  t = MPStrDup(mp, xlfd);
  t++;
  fields[0] = t;
  for (i = 1; i < XLFD_COUNT; t++)
  {
    if (*t == '-')
    {
      *t = '\0';
      fields[i++] = t + 1;
    }
  }

  if (strcmp(fields[XLFD_WEIGHT_NAME], "bold") == 0) lfi->weight = 1;
  else lfi->weight = 0;

  lfi->size = atoi(fields[XLFD_PIXEL_SIZE]);

  if (strcmp(fields[XLFD_SLANT], "i") == 0 ||
      strcmp(fields[XLFD_SLANT], "o") == 0)
  {
    lfi->slant = 1;
  }
  else lfi->slant = 0;

  if (strcmp(fields[XLFD_SPACING], "p") == 0) lfi->fixed = false;
  else lfi->fixed = true;

  return;
}

/*
 * GetFont
 */
static XFontStruct *
GetFont(lc, li, i)
LayoutClass *lc;
LayoutInfo *li;
int i;
{
  if (lc->fontInfo[i].xfi == NULL)
  {
    lc->fontInfo[i].xfi = XLoadQueryFont(li->dpy, lc->fontNames[i]);
    if (lc->fontInfo[i].xfi == NULL) lc->fontInfo[i].xfi = lc->defaultFont;
  }
  return(lc->fontInfo[i].xfi);
}

/*
 * LayoutGetFont
 */
XFontStruct *
LayoutGetFont(li)
LayoutInfo *li;
{
  int i;
  LayoutClass *lc = li->lc;
  LayoutFontInfo *lfi;

  lfi = &(li->restack->fi);

  /*
   * Check spacing
   */
  if (lfi->fixed) return(lc->fixedFont);

  /*
   * Check slant, size, and weight
   */
  for (i = 0; i < lc->fontCount; i++)
  {
    if (((lfi->weight > 0) == (lc->fontInfo[i].weight > 0)) &&
	((lfi->slant > 0) == (lc->fontInfo[i].slant > 0)) &&
	lc->scale[lfi->scale] == lc->fontInfo[i].size)
    {
      return(GetFont(lc, li, i));
    }
  }

  /*
   * Check slant and size
   */
  for (i = 0; i < lc->fontCount; i++)
  {
    if (((lfi->slant > 0) == (lc->fontInfo[i].slant > 0)) &&
	lc->scale[lfi->scale] == lc->fontInfo[i].size)
    {
      return(GetFont(lc, li, i));
    }
  }

  /*
   * Check weight and size
   */
  for (i = 0; i < lc->fontCount; i++)
  {
    if (((lfi->weight > 0) == (lc->fontInfo[i].weight > 0)) &&
	lc->scale[lfi->scale] == lc->fontInfo[i].size)
    {
      return(GetFont(lc, li, i));
    }
  }

  /*
   * Check size
   */
  for (i = 0; i < lc->fontCount; i++)
  {
    if (lc->scale[lfi->scale] == lc->fontInfo[i].size)
    {
      return(GetFont(lc, li, i));
    }
  }

  return(lc->defaultFont);
}

/*
 * LayoutSetupFonts
 */
void
LayoutSetupFonts(li)
LayoutInfo *li;
{
  char *name;
  int i, count, scale_count;
  LayoutClass *lc = li->lc;
  MemPool mp;

  /*
   * Default HTML font.
   */
  li->cfi.fixed = false;
  li->cfi.weight = 0;
  li->cfi.slant = 0;
  li->cfi.scale = 2;
  li->cfi.size = 0;
  li->cfi.xfi = NULL;

  /*
   * Only need to do HTML font information setup once.
   */
  if (lc->font_setup_done) return;

  /*
   * Font to use for plain text and teletype stuff.
   */
  name = FrameGetResource(li->frame, "html.fixedFont");
  if (name == NULL) name = "fixed";
  if ((lc->fixedFont = XLoadQueryFont(li->dpy, name)) == NULL)
  {
    fprintf (stderr, "Could not get fixed font.\n");
    fflush(stderr);
    exit(1);
  }

  /*
   * Font to use if all else fails
   */
  name = FrameGetResource(li->frame, "html.defaultFont");
  if (name == NULL) name = "variable";
  if ((lc->defaultFont = XLoadQueryFont(li->dpy, name)) == NULL)
  {
    fprintf (stderr, "Could not get default font.\n");
    fflush(stderr);
    exit(1);
  }

  /*
   * Get the font list pattern for the proportional fonts.  Hopefully
   * a reasonable pattern was selected.
   */
  name = FrameGetResource(li->frame, "html.propFontPattern");
  if (name == NULL) name = "-adobe-times-*-*-*-*-*-*-100-100-*-*-iso8859-1";

  lc->fontNames = XListFonts(li->dpy, name, 999, &count);
  if (lc->fontNames == NULL || count == 0)
  {
    fprintf (stderr, "Could not get font list\n");
    fflush(stderr);
    exit(1);
  }

  lc->fontInfo = (LayoutFontInfo *)alloc_mem(sizeof(LayoutFontInfo) * count);
  memset(lc->fontInfo, 0, sizeof(LayoutFontInfo) * count);

  /*
   * Extract simple information from the XLFD strings.
   */
  mp = MPCreate();
  for (i = 0; i < count; i++)
  {
    ParseXLFD(mp, lc, lc->fontNames[i], &(lc->fontInfo[i]));
  }
  MPDestroy(mp);

  /*
   * Find sizes for each of the font scales.  There must be a better way.
   * This should probably not take 
   */
  scale_count = 0;
  for (i = 0; i < count && scale_count < FONT_SCALE_COUNT; i++)
  {
    if (lc->fontInfo[i].size > 0)
    {
      int j;
      for (j = 0; j < scale_count; j++)
      {
	if (lc->fontInfo[i].size == lc->scale[j]) break;
	if (lc->fontInfo[i].size < lc->scale[j])
	{
	  int k;
	  for (k = scale_count - 1; k >= j; k--)
	  {
	    lc->scale[k + 1] = lc->scale[k];
	  }
	  lc->scale[j] = lc->fontInfo[i].size;
	  scale_count++;
	  break;
	}
      }
      if (j == scale_count)
      {
	lc->scale[j] = lc->fontInfo[i].size;
	scale_count++;
      }
    }
  }

  /*
   * This will probably never happen.
   */
  if (scale_count == 0)
  {
    lc->scale[0] = lc->fontInfo[0].size;
    scale_count++;
  }
  
  /*
   * Fill out the rest if there aren't FONT_SCALE_COUNT
   */
  if (scale_count < FONT_SCALE_COUNT)
  {
    for (i = scale_count; i < FONT_SCALE_COUNT; i++)
    {
      lc->scale[i] = lc->scale[i - 1];
    }
    scale_count = FONT_SCALE_COUNT;
  }

  /*
   * Set miscellaneous information.
   */
  lc->fontCount = count;
  lc->font_setup_done = true;

  return;
}
