/*
 * list.c
 *
 * Copyright (c) 1995, John D. Kilburg (john@cs.unlv.edu)
 * See the file COPYRIGHT for more information.
 */
#include <stdio.h>

#include "layout.h"

/*
 *
 * Private functions
 *
 */
static void RenderLI _ArgProto((LayoutInfo *, RenderPart *));
static void SetupLI _ArgProto((LayoutInfo *, RenderPart *, int, int));

/*
 * RenderLI
 */
static void
RenderLI(li, rp)
LayoutInfo *li;
RenderPart *rp;
{
  XFillArc(li->dpy, li->win, li->gc,
	   rp->x, rp->y, rp->width, rp->height,
	   0, 360 * 64);

  return;
}

/*
 * SetupLI
 */
static void
SetupLI(li, rp, line_y, line_height)
LayoutInfo *li;
RenderPart *rp;
int line_y;
int line_height;
{
  rp->y = line_y + line_height / 2;
  rp->renderable = true;
  return;
}

bool
InListItemEnv(li, item)
LayoutInfo *li;
int *item;
{
  RenderEnv *re;

  for (re = li->restack; re != NULL; re = re->next)
  {
    if (re->tagid == TAG_UL || re->tagid == TAG_DL || re->tagid == TAG_OL ||
	re->tagid == TAG_MENU || re->tagid == TAG_DIR)
    {
      return(false);
    }
    else if (re->tagid == TAG_LH || re->tagid == TAG_LI ||
	     re->tagid == TAG_DT || re->tagid == TAG_DD)
    {
      *item = re->tagid;
      return(true);
    }
  }

  return(false);
}

/*
 *
 * Public functions
 *
 */
void
HandleList(li, tagid, p, end)
LayoutInfo *li;
int tagid;
MLTag *p;
int end;
{
  if (end != 0)
  {
    AddLineBreak(li);
    DestroyRenderEnv(li, tagid);
    AddLineBreak(li);
  }
  else
  {
    RenderEnv *re;

    AddLineBreak(li);
    re = CreateRenderEnv(li, p->id);
    if (tagid != TAG_DL)
    {
      re->x += 20;
      re->width -= 20;
    }
    AddLineBreak(li);
  }

  return;
}

void
HandleLI(li, p, end)
LayoutInfo *li;
MLTag *p;
int end;
{
  RenderPart *rp;
  XFontStruct *font;
  int item;
  RenderEnv *re;

  if (end != 0)
  {
    DestroyRenderEnv(li, TAG_NOTHING);
    DestroyRenderEnv(li, TAG_LI);
  }
  else
  {
    if (InListItemEnv(li, &item)) DestroyRenderEnv2(li, item, true);

    CreateRenderEnv(li, TAG_LI);
    font = LayoutGetFont(li);
    AddLineBreak(li);
    
    rp = CreateRenderPart(li, p, SetupLI, RenderLI, NULL, NULL, 0);
    rp->breakable = false;
    AddRenderPart(li, rp, 7, 7);

    rp = CreateRenderPart(li, NULL, NULL, NULL, NULL, NULL, 0);
    rp->breakable = false;
    AddRenderPart(li, rp, XTextWidth(font, " ", 1),
		  font->ascent + font->descent);

    re = CreateRenderEnv(li, TAG_NOTHING);
    re->width -= (li->rltail->x + li->rltail->width) - re->x;
    re->x = li->rltail->x + li->rltail->width;
  }

  return;
}

void
HandleOL(li, p, end)
LayoutInfo *li;
MLTag *p;
int end;
{
  int item;

  if (end != 0) DestroyRenderEnv(li, TAG_OL);
  else
  {
    if (InListItemEnv(li, &item)) DestroyRenderEnv2(li, item, true);
    CreateRenderEnv(li, TAG_OL);
    AddLineBreak(li);
  }
 
  return;
}

void
HandleDD(li, p, end)
LayoutInfo *li;
MLTag *p;
int end;
{
  int item;
  RenderEnv *re;

  if (end != 0) DestroyRenderEnv(li, TAG_DD);
  else
  {
    if (InListItemEnv(li, &item)) DestroyRenderEnv2(li, item, true);
    re = CreateRenderEnv(li, TAG_DD);
    re->x += 20;
    re->width -= 20;

    AddLineBreak(li);
  }

  return;
}

void
HandleDT(li, p, end)
LayoutInfo *li;
MLTag *p;
int end;
{
  int item;

  if (end != 0) DestroyRenderEnv(li, TAG_DT);
  else
  {
    if (InListItemEnv(li, &item)) DestroyRenderEnv2(li, item, true);
    CreateRenderEnv(li, TAG_DT);
    AddLineBreak(li);
  }

  return;
}

