/*
 * misc.c
 *
 * Copyright (c) 1995, John D. Kilburg (john@isri.unlv.edu)
 * See the file COPYRIGHT for details.
 */
#include <stdio.h>
#include <ctype.h>

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include "layout.h"

void
HandleEOF(li)
LayoutInfo *li;
{
  AddBlankLine(li);

  return;
}

void
HandleA(li, p, end)
LayoutInfo *li;
MLTag *p;
int end;
{
  if (end != 0) DestroyRenderEnv(li, TAG_A);
  else
  {
    RenderEnv *re;

    if (MLFindAttribute(p, "href") != NULL)
    {
      if (InEnv(li, TAG_A)) DestroyRenderEnv2(li, TAG_A, true);
      
      re = CreateRenderEnv(li, TAG_A);
      
      if (MLFindAttribute(p, "href") != NULL) re->anchorPart = p;
      if (MLFindAttribute(p, "name") != NULL) re->namePart = p;
    }
    else if (MLFindAttribute(p, "name") != NULL)
    {
      RenderPart *rp;

      rp = CreateRenderPart(li, p, NULL, NULL, NULL, NULL, 0);
      rp->namePart = p;
      AddRenderPart(li, rp, 0, 0);
    }
  }

  return;
}

void
HandleHx(li, tag, p, end)
LayoutInfo *li;
int tag;
MLTag *p;
int end;
{
  RenderEnv *re;

  if (end != 0)
  {
    AddBlankLine(li);
    DestroyRenderEnv(li, tag);
    AddBlankLine(li);
  }
  else
  {
    char *align;

    AddBlankLine(li);

    re = CreateRenderEnv(li, tag);
    if (p->id == TAG_H1) re->fi.scale = 5;
    else if (p->id == TAG_H2) re->fi.scale = 4;
    else if (p->id == TAG_H3) re->fi.scale = 3;
    else if (p->id == TAG_H4) re->fi.scale = 2;
    else if (p->id == TAG_H5) re->fi.scale = 1;
    else if (p->id == TAG_H6) re->fi.scale = 0;

    if ((align = MLFindAttribute(p, "align")) != NULL)
    {
      re->align = MLGetAlignID(align);
    }

    AddBlankLine(li);
  }
  
  return;
}

void
HandleFontChange(li, end, tag)
LayoutInfo *li;
int end;
int tag;
{

  if (end != 0) DestroyRenderEnv(li, tag);
  else
  {
    RenderEnv *re;

    re = CreateRenderEnv(li, tag);
    if (tag == TAG_EM || tag == TAG_CITE || tag == TAG_I) re->fi.slant++;
    else if (tag == TAG_B || tag == TAG_STRONG) re->fi.weight++;
    else if (tag == TAG_TT) re->fi.fixed = true;
  }
  
  return;
}

void
HandleP(li, p, end)
LayoutInfo *li;
MLTag *p;
int end;
{
  RenderEnv *re;

  if (end != 0)
  {
    AddLineBreak(li);
    DestroyRenderEnv(li, TAG_P);
  }
  else
  {
    char *align;

    re = CreateRenderEnv(li, TAG_P);
    if ((align = MLFindAttribute(p, "align")) != NULL)
    {
      re->align = MLGetAlignID(align);
    }
    AddBlankLine(li);
    AddBlankLine(li);
  }

  return;
}

/*
 * HandleBR
 *
 * This is function is evil.  It does things differently.
 */
void
HandleBR(li, p, end)
LayoutInfo *li;
MLTag *p;
int end;
{
  if (end == 0)
  {
    char *align;

    if ((align = MLFindAttribute(p, "align")) != NULL)
    {
      li->rltail->align = MLGetAlignID(align);
    }

    AddLineBreak(li);
  }

  return;
}

void
HandleAddress(li, p, end)
LayoutInfo *li;
MLTag *p;
int end;
{
  if (end != 0)
  {
    DestroyRenderEnv(li, TAG_ADDRESS);
    AddLineBreak(li);
  }
  else
  {
    RenderEnv *re;

    if (InEnv(li, TAG_ADDRESS)) return;

    AddLineBreak(li);
    re = CreateRenderEnv(li, TAG_ADDRESS);
    re->fi.slant++;
  }

  return;
}


void
HandleBase(li, p, end)
LayoutInfo *li;
MLTag *p;
int end;
{
  char *url;
  URLParts *up;

  if (end == 0)
  {
    if ((url = MLFindAttribute(p, "href")) != NULL)
    {
      if ((up = URLParse(li->mp, url)) != NULL) li->bup = up;
    }
  }

  return;
}

void
HandleMisc(li, tagid, p, end)
LayoutInfo *li;
int tagid;
MLTag *p;
int end;
{
  if (end == 0) CreateRenderEnv(li, tagid);
  else DestroyRenderEnv(li, tagid);

  return;
}

void
HandleBlockquote(li, p, end)
LayoutInfo *li;
MLTag *p;
int end;
{
  if (end != 0)
  {
    AddBlankLine(li);
    DestroyRenderEnv(li, TAG_BLOCKQUOTE);
    AddBlankLine(li);
  }
  else
  {
    RenderEnv *re;

    if (InEnv(li, TAG_BLOCKQUOTE)) DestroyRenderEnv2(li, TAG_BLOCKQUOTE, true);

    AddBlankLine(li);
    re = CreateRenderEnv(li, TAG_BLOCKQUOTE);
    re->x += re->width * 10 / 100;
    re->width -= re->width * 30 / 100;
    re->fi.slant++;
    AddBlankLine(li);
  }

  return;
}
