/*
 * module.c
 *
 * Copyright (c) 1995, John D. Kilburg <john@cs.unlv.edu>
 * See the file COPYRIGHT for details.
 */
#include <stdio.h>

#include "layout.h"

static int
LayoutAddData(closure, d)
void *closure;
Document *d;
{
  LayoutInfo *li = (LayoutInfo *)closure;

  li->d = d;
  MLAddData(li->hs, (char *)d->data, d->len);

  return(0);
}

static int
LayoutEndData(closure, d)
void *closure;
Document *d;
{
  LayoutInfo *li = (LayoutInfo *)closure;

  li->d = d;
  MLEndData(li->hs, (char *)d->data, d->len);

  return(0);
}

static int
LayoutExpose(closure, x, y, width, height)
void *closure;
int x, y;
unsigned int width, height;
{
  LayoutInfo *li = (LayoutInfo *)closure;
  RenderLine *rl;
  RenderPart *rp;
  int rly2;
  int y1 = y;
  int y2 = y1 + height;
  Region r;
  XRectangle rec;

  r = XCreateRegion();
  rec.x = (short)x;
  rec.y = (short)y;
  rec.width = (unsigned short)width;
  rec.height = (unsigned short)height;
  XUnionRectWithRegion(&rec, r, r);

  for (rl = li->rlhead; rl != NULL; rl = rl->next)
  {
    rly2 = rl->y + rl->height;
    if (rl->y < y1 && rly2 < y1) continue;
    else if (rl->y > y2 && rly2 > y2) break;

    for (rp = rl->rphead; rp != NULL; rp = rp->lnext)
    {
      if (IsRenderable(rp) && rp->render != NULL)
      {
	if (XRectInRegion(r, rp->x, rp->y, rp->width, rp->height))
	{
	  (*(rp->render))(li, rp);
	}
      }
    }
  }

  XDestroyRegion(r);

  return(0);
}

/*
 * InAnchor
 */
static bool
InAnchor(x, y, rp)
int x, y;
RenderPart *rp;
{
  if (x > rp->x &&
      x < rp->x + rp->width &&
      y > rp->y + rp->height - rp->height &&
      y < rp->y + rp->height)
  {
    return(true);
  }

  return(false);
}

static int
LayoutMouse(closure, x, y, button)
void *closure;
int x, y;
int button;
{
  LayoutInfo *li = (LayoutInfo *)closure;
  char *url, *aurl;
  RenderPart *rp;

  for (rp = li->aphead; rp != NULL; rp = rp->anext)
  {
    if (InAnchor(x, y, rp))
    {
      if ((url = MLFindAttribute(rp->anchorPart, "href")) != NULL)
      {
	URLParts *lup;
	MemPool mp;

	mp = MPCreate();
	if (rp->p != NULL && MLFindAttribute(rp->p, "ismap") != NULL)
	{
	  aurl = (char *)MPGet(mp, strlen(url) + 100 + 100);
	  sprintf (aurl, "%s?%d,%d", url, x - rp->x, y - rp->y);
	  url = aurl;
	}
	else aurl = NULL;

	if ((lup = URLParse(mp, url)) != NULL)
	{
	  URLParts *up;
	  
	  if (li->bup != NULL) up = URLResolve(mp, lup, li->bup);
	  else up = lup;

	  /*
           * Careful with these two...may lose LayoutInfo before they
           * return.
           */
	  if (button == 1) FrameLoad(li->frame, up);
	  else if (button == 2) FrameDownload(li->frame, up);
	}
	MPDestroy(mp);
      }
      break;
    }
  }

  return(0);
}

static int
LayoutMotion(closure, x, y, button)
void *closure;
int x, y;
int button;
{
  LayoutInfo *li = (LayoutInfo *)closure;
  char *url;
  RenderPart *rp;
  
  for (rp = li->aphead; rp != NULL; rp = rp->anext)
  {
    if (InAnchor(x, y, rp)) break;
  }

  if (li->over == rp) return(0);

  if (rp != NULL)
  {
    if ((url = MLFindAttribute(rp->anchorPart, "href")) != NULL)
    {
      URLParts *lup;
      MemPool mp;

      mp = MPCreate();
      if ((lup = URLParse(mp, url)) != NULL)
      {
	URLParts *up;
	char *turl;
	
	if (li->bup != NULL) up = URLResolve(mp, lup, li->bup);
	else up = lup;
	
	turl = URLMakeString(mp, up, 0);
	FrameMessage(li->frame, turl);
      }
      MPDestroy(mp);
    }
  }
  else FrameMessage(li->frame, NULL);

  li->over = rp;

  return(0);
}

/*
 * LayoutClean
 */
void
LayoutClean(li)
LayoutInfo *li;
{
  if (li == NULL) return;

  DestroyRenderEnv(li, TAG_START);

  if (li->rphead != NULL)
  {
    RenderPart *t, *c;
    
    for (c = li->rphead; c != NULL; )
    {
      t = c;
      c = c->mnext;
      DestroyRenderPart(t);
    }
  }

  if (li->title != NULL) free_mem(li->title);
  if (li->author != NULL) free_mem(li->author);
  if (li->hs != NULL) MLDestroy(li->hs);
  if (li->mp != NULL) MPDestroy(li->mp);

  return;
}

static int
LayoutDestroy(closure)
void *closure;
{
  LayoutInfo *li = (LayoutInfo *)closure;

  LayoutClean(li);
  free_mem(li);

  return(0);
}

/*
 * LayoutInit
 */
static void *
LayoutInit(up, d, lc, frame)
URLParts *up;
Document *d;
LayoutClass *lc;
Frame frame;
{
  RenderEnv *re;
  LayoutInfo *li;
  char *content;

  li = (LayoutInfo *)alloc_mem(sizeof(LayoutInfo));
  memset(li, 0, sizeof(LayoutInfo));
  li->mp = MPCreate();
  li->bup = URLDup(li->mp, up);
  li->delay = false;
  li->destroy_locked = false;
  li->line_space = 2;
  li->frame = frame;
  li->d = d;
  li->hs = MLInit(li->mp, true, LayoutHandler, li);
  li->lc = lc;

  /*
   * Graphics setup
   */
  li->w = (Widget)FrameDrawWidget(frame);
  li->dpy = XtDisplay(li->w);
  li->win = XtWindow(li->w);
  li->gc = DefaultGC(li->dpy, DefaultScreen(li->dpy));

  FrameGetNamedColor(frame, "blue", &(li->anchor_color));
  FrameGetNamedColor(frame, "black", &(li->fg));

  LayoutSetupFonts(li);

  /*
   * Get the size of the window...if the size is unknown then
   * just use 150x150.
   */
  if (FrameGetSize(frame, true, &(li->width), &(li->height)) == -1)
  {
    li->width = 200;   
    li->height = 200;
    FrameReturnSize(frame, true, li->width, li->height);
  }

  /*
   * Setup environment
   */
  re = CreateRenderEnv(li, TAG_START);

  /*
   * Intialize line queue
   */
  AddLineBreak(li);

  /*
   * Fake a tag for plain text.
   */
  content = MIMEGetField(d->mflist, "content-type");
  if (content != NULL && strcmp(content, "text/plain") == 0)
  {
    HandlePlaintext(li, NULL, 0);
  }

  return(li);
}

/*
 * LayoutReset
 */
static int
LayoutReset(closure)
void *closure;
{
  return(0);
}

int
InitModule_HTML(wc)
WWWContext wc;
{
  static RenderHooks rh[2];
  LayoutClass *lc;

  lc = (LayoutClass *)alloc_mem(sizeof(LayoutClass));
  memset(lc, 0, sizeof(LayoutClass));

  memset(&rh[0], 0, sizeof(RenderHooks));
  rh[0].class_context = lc;
  rh[0].content = "text/html";
  rh[0].init = LayoutInit;
  rh[0].reset = LayoutReset;
  rh[0].add = LayoutAddData;
  rh[0].end = LayoutEndData;
  rh[0].expose = LayoutExpose;
  rh[0].mouse = LayoutMouse;
  rh[0].destroy = LayoutDestroy;
  rh[0].motion = LayoutMotion;
  WWWAddRenderHooks(wc, &rh[0]);

  memset(&rh[1], 0, sizeof(RenderHooks));
  rh[1].class_context = lc;
  rh[1].content = "text/plain";
  rh[1].init = LayoutInit;
  rh[1].reset = LayoutReset;
  rh[1].add = LayoutAddData;
  rh[1].end = LayoutEndData;
  rh[1].expose = LayoutExpose;
  rh[1].mouse = LayoutMouse;
  rh[1].destroy = LayoutDestroy;
  rh[1].motion = LayoutMotion;
  WWWAddRenderHooks(wc, &rh[1]);

  return(0);
}
