/*
 * jpegp.h
 *
 * Copyright (C) 1995, Erik Corry (ehcorry@inet.uni-c.dk)
 * See the file COPYRIGHT for details
 */

/*
 * This stuff is very fragile wrt. changes in the sizes of the
 * jpeg library structures. Binary compatibility is a Bohemian
 * village.
 */

#ifndef JPEG_H_INCLUDED
#define JPEG_H_INCLUDED 1

#include <sys/types.h>
#include <stdio.h>
#include <setjmp.h>              /* ANSI 7.6.2.1 */
#include "jpeglib.h"
#include "jerror.h"
#include "image_format.h"

struct jpeg_chimera_error_state
{
  struct jpeg_error_mgr pub; /* - this struct comes from jpeg lib */
  jmp_buf jump_buffer;       /* usually an array of sorts from setjmp.h */
};

struct jpeg_chimera_input_state
{
  struct jpeg_source_mgr pub;   /* - this struct comes from jpeg lib */
  byte *data;              /* Compressed input data buffer */
  int len;                 /* How much data is in the buffer */
  int old_len;             /* This is how much we have told the jpeglib about */
};

/*
 * jpegState
 */
typedef struct jpeg_state
{
  int state;               /* state of JPEG reader */
  FormatLineProc lineProc; /* line callback */
  void *closure;           /* closure for callback */

  Image *image;
  Intensity cmap[256];

  int   pos;               /* current read position */
  int   ypos;              /* current line */
  byte *imagepos;          /* current posn. in output */

  bool   destroy_jpeg;
  struct jpeg_decompress_struct cinfo; /* struct from jpeg lib */
  struct jpeg_error_mgr jerr;          /* struct from jpeg lib */

  struct jpeg_chimera_input_state input_state;
  struct jpeg_chimera_error_state error_state;

} jpegState;

/*
 * state of JPEG reader
 */
#define CH_JPEG_READ_HEADER 1
#define CH_JPEG_START_DECOMPRESS 2
#define CH_JPEG_START_OUTPUT 3
#define CH_JPEG_READ_IMAGE 4
#define CH_JPEG_FINISH_DECOMPRESS 5
#define CH_JPEG_FINISHED 0

/*
 * return values from jpeg processing fns
 */
#define CH_JPEG_ERROR 0
#define CH_JPEG_SUCCESS 1
#define CH_JPEG_NEED_DATA 2

#endif
