/*
 * message.c
 *
 * Copyright (c) 1995, John Kilburg <john@cs.unlv.edu>
 * See the file COPYRIGHT for details.
 */
#include <stdio.h>

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include "WWW.h"

typedef struct _message_info
{
  MemPool mp;
  Frame frame;
  URLParts *up;
  IOTimeOut iot;
} MessageInfo;

static void
MessageRead(iot, mi)
IOTimeOut iot;
MessageInfo *mi;
{
  char *text;
  Document *d;
  char *name;

  name = (char *)alloc_mem(strlen(mi->up->filename) + strlen("message.") + 1);
  strcpy(name, "message.");
  strcat(name, mi->up->filename);

  text = FrameGetResource(mi->frame, name);
  if (text == NULL) text = name;
  else text = alloc_string(text);

  d = DocumentCreate(true);
  d->ds = d->data = (byte *)text;
  d->len = strlen(text);
  d->mflist = MIMECreateField(d->mp, "content-type", "text/html");
  d->nocache = true;

  FrameInitData(mi->frame, d);
  FrameEndData(mi->frame, d);

  return;
}

static void *
MessageInit(up, pup, class_context, frame)
URLParts *up;
URLParts *pup;
void *class_context;
Frame frame;
{
  MessageInfo *mi;
  MemPool mp;

  mp = MPCreate();
  mi = (MessageInfo *)MPCGet(mp, sizeof(MessageInfo));
  mi->mp = mp;
  mi->frame = frame;
  mi->up = URLDup(mi->mp, up);
  mi->iot = IOCreateTimeOut(frame, mi);

  IOSetTimeOut(mi->iot, 10, MessageRead);

  return(mi);
}

static int
MessageDestroy(mi)
MessageInfo *mi;
{
  if (mi->iot != NULL) IODestroyTimeOut(mi->iot);
  MPDestroy(mi->mp);
  return(0);
}

void
InitModule_Message(wc)
WWWContext wc;
{
  static ProtoHooks ph;

  ph.protocol = "x-message";
  ph.init = MessageInit;
  ph.destroy = MessageDestroy;
  WWWAddProtoHooks(wc, &ph);

  return;
}
