/*
 * main.c
 *
 * Copyright (C) 1993, 1994, 1995, John D. Kilburg (john@cs.unlv.edu)
 *
 * See copyright.h for details.
 */
#include "copyright.h"

#include <stdio.h>
#include <ctype.h>
#include <signal.h>
#include <sys/param.h>

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>

#include "WWW.h"
#include "StrReq.h"
#include "resources.h"

static void sigigh_handler _ArgProto(());
static void xtWarningHandler _ArgProto((String));
static void xtErrorHandler _ArgProto((String));
static int xErrorHandler _ArgProto((Display *, XErrorEvent *));
static void MakeHead _ArgProto((Widget, char *));
static void DeleteAction(), ReturnAction();

extern void InitModule_Setup();

AppResources appr;

/*
 * These are initialized in fallback.c
 */
#define offset(field) XtOffset(AppResources *, field)
static XtResource       resource_list[] =
{
  { "homeURL", "URL", XtRString, sizeof(char *),
	offset(homeURL), XtRString, NULL },
  { "helpURL", "URL", XtRString, sizeof(char *),
	offset(helpURL), XtRString, NULL },
  { "button1Box", "BoxList", XtRString, sizeof(char *),
	offset(button1Box), XtRString, NULL },
  { "button2Box", "BoxList", XtRString, sizeof(char *),
	offset(button2Box), XtRString, NULL },
  { "dbFiles", XtCFile, XtRString, sizeof(char *),
	offset(dbFiles), XtRString, NULL },
  { "messageFont", XtCFont, XtRFontStruct, sizeof(XFontStruct *),
	offset(messageFont), XtRString, NULL },
};

extern char *fallback_resources[];

static XtActionsRec actionsList[] =
{
  { "ReturnAction", (XtActionProc)ReturnAction },
  { "DeleteAction",  (XtActionProc)DeleteAction  },
};

/*
 * main
 *
 * this is where the whole thing kicks off.
 */
void
main(argc, argv)
int argc;
char **argv;
{
  XGCValues xgcv;
  Widget toplevel;
  MemPool mp;
  char *first, *rfirst;

  memset(&appr, 0, sizeof(appr));

  appr.argv0 = argv[0];

#ifdef SOCKS
  SOCKSinit(argv[0]);
#endif

  signal(SIGINT, sigigh_handler);
  signal(SIGQUIT, sigigh_handler);
  signal(SIGHUP, sigigh_handler);
  signal(SIGTERM, sigigh_handler);
  signal(SIGPIPE, SIG_IGN);

  StartReaper();

  toplevel = XtAppInitialize(&(appr.appcon), "Chimera", NULL, 0,
			     &argc, argv, fallback_resources, NULL, 0);
  if (toplevel == 0) exit(1);

  XSetErrorHandler(xErrorHandler);
  XtAppSetErrorHandler(appr.appcon, xtErrorHandler);
  XtAppSetWarningHandler(appr.appcon, xtWarningHandler);

  appr.dpy = XtDisplay(toplevel);

  XtGetApplicationResources(toplevel, &appr, resource_list,
			    XtNumber(resource_list), NULL, 0);

  appr.wc = WWWCreateContext(NULL, NULL);
  WWWAddResourceFile(appr.wc, appr.dbFiles);

  if (appr.messageFont != NULL)
  {
    xgcv.font = appr.messageFont->fid;
    xgcv.foreground = BlackPixel(appr.dpy, DefaultScreen(appr.dpy));
    appr.gc = XCreateGC(appr.dpy, DefaultRootWindow(appr.dpy),
			GCFont | GCForeground, &xgcv);
  }

  XtAppAddActions(appr.appcon, actionsList, XtNumber(actionsList));

  InitModule_Setup(appr.wc);

  mp = MPCreate();

  if (argc > 1) rfirst = argv[1];
  else rfirst = getenv("WWW_HOME");
  if (rfirst == NULL) rfirst = appr.homeURL;
  if (rfirst == NULL) rfirst = "file:/";

  first = (char *)MPGet(mp, strlen(rfirst) + strlen("chimera-setup:") + 1);
  strcpy(first, "chimera-setup:");
  strcat(first, rfirst);
      
  MakeHead(toplevel, first);

  MPDestroy(mp);

  XtAppMainLoop(appr.appcon);
}

/*
 * DeleteAction
 */
static void
DeleteAction()
{
  while (appr.hlist != NULL)
  {
    DestroyHead(&appr, appr.hlist);
  }
  exit(0);
}

/*
 * ReturnAction
 */
static void
ReturnAction(w, xe, params, num_params)
Widget w;
XEvent *xe;
String *params;
Cardinal *num_params;
{
  WWWHead *c;
  char *url;
  URLParts *up;

  for (c = appr.hlist; c != NULL; c = c->next)
  {
    if (c->turl == w)
    {
      XtVaGetValues(c->turl, XtNstring, &url, NULL);
      if (url == NULL || url[0] == '\0')
      {
	XtVaSetValues(c->turl, XtNstring, c->surl, NULL);
	break;
      }
      else
      {
	MemPool mp = MPCreate();
	if ((up = URLParse(mp, url)) != NULL) FrameLoad(c->frame, up);
	MPDestroy(mp);
      }
      break;
    }
  }

  return;
}

/*
 * MakeHead
 */
static void
MakeHead(toplevel, first)
Widget toplevel;
char *first;
{
  Atom delete;
  WWWHead *h;
  URLParts *up;
  MemPool mp;

  h = CreateHead(&appr, toplevel);

  XtRealizeWidget(toplevel);

  delete = XInternAtom(XtDisplay(toplevel), "WM_DELETE_WINDOW", False);
  XtOverrideTranslations (toplevel, 
			  XtParseTranslationTable
			  ("<Message>WM_PROTOCOLS: DeleteAction()"));
  XSetWMProtocols (appr.dpy, XtWindow(toplevel), &delete, 1);

  XtOverrideTranslations(h->turl,
			 XtParseTranslationTable
			 ("<Key>Return: ReturnAction()"));

  if (first == NULL) first = getenv("WWW_HOME");
  if (first == NULL) first = appr.homeURL;

  mp = MPCreate();
  if ((up = URLParse(mp, first)) != NULL) FrameLoad(h->frame, up);
  MPDestroy(mp);

  return;
}

/*
 * ClearMessage
 */
static void
ClearMessage(h)
WWWHead *h;
{
  XClearWindow(XtDisplay(h->message), XtWindow(h->message));

  return;
}

/*
 * SetMessage
 */
static void
SetMessage(h, str)
WWWHead *h;
char *str;
{
  Display *dpy;

  ClearMessage(h);

  dpy = XtDisplay(h->message);
  XDrawString(dpy, XtWindow(h->message), appr.gc,
	      10, appr.messageFont->ascent + 2, str, strlen(str));

  return;
}

/*
 * MessageCallback
 */
void
MessageCallback(frame, cldata, cbdata)
Frame frame;
void *cldata;
char *cbdata;
{
  if (cbdata == NULL) ClearMessage((WWWHead *)cldata);
  else SetMessage((WWWHead *)cldata, cbdata);

  return;
}

/*
 * URLCallback
 */
void
URLCallback(frame, cldata, cbdata)
Frame frame;
void *cldata;
char *cbdata;
{
  WWWHead *h = (WWWHead *)cldata;
  char *s;

  if (h->surl != NULL) free_mem(h->surl);
  if (cbdata == NULL)
  {
    h->surl = NULL;
    s = "";
  }
  else s = h->surl = alloc_string(cbdata);

  XtVaSetValues(h->turl, XtNstring, s, NULL);

  return;
}

/*
 * OOCallback
 */
static void
OOCallback(w, cldata, cbdata)
Widget w;
XtPointer cldata, cbdata;
{
  WWWHead *h = (WWWHead *)cldata;
  URLParts *up;
  MemPool mp;

  if (cbdata == NULL) return;
  mp = MPCreate();
  if ((up = URLParse(mp, (char *)cbdata)) != NULL) FrameLoad(h->frame, up);
  MPDestroy(mp);
  XtPopdown(w); 

  return;
}

/*
 * Open
 */
void
Open(w, cldata, cbdata)
Widget w;
XtPointer cldata, cbdata;
{
  WWWHead *h = (WWWHead *)cldata;

  if (h->openpop == NULL)
  {
    Window rw, cw;
    int rx, ry, wx, wy;
    unsigned int mask;

    XQueryPointer(appr.dpy, DefaultRootWindow(appr.dpy),
		  &rw, &cw,
		  &rx, &ry,
		  &wx, &wy,
		  &mask);
    h->openpop = XtVaCreatePopupShell("openurl",
				      strReqWidgetClass, h->toplevel,
				      XtNdefaultString, NULL,
				      XtNhiddenText, False,
				      XtNx, rx - 2,
				      XtNy, ry - 2,
				      NULL);
    XtAddCallback(h->openpop, XtNokCallback, OOCallback, h);
    XtRealizeWidget(h->openpop);
  }

  XtPopup(h->openpop, XtGrabNone);

  return;
}

/*
 * Back
 */
void
Back(w, cldata, cbdata)
Widget w;
XtPointer cldata, cbdata;
{
  WWWHead *h = (WWWHead *)cldata;
  FrameBack(h->frame);
  return;
}

/*
 * Reload
 */
void
Reload(w, cldata, cbdata)
Widget w;
XtPointer cldata, cbdata;
{
  WWWHead *h = (WWWHead *)cldata;
  FrameReload(h->frame);
  return;
}

/*
 * Cancel
 */
void
Cancel(w, cldata, cbdata)
Widget w;
XtPointer cldata, cbdata;
{
  WWWHead *h = (WWWHead *)cldata;
  FrameCancel(h->frame);
  return;
}

/*
 * Quit
 */
void
Quit(w, cldata, cbdata)
Widget w;
XtPointer cldata, cbdata;
{
  DestroyHead(&appr, (WWWHead *)cldata);
  if (appr.hlist == NULL)
  {
    WWWDestroyContext(appr.wc);
/*    (void)MemPoolExisting(); */
    exit(0);
  }
  return;
}

/*
 * Home
 */
void
Home(w, cldata, cbdata)
Widget w;
XtPointer cldata, cbdata;
{
  WWWHead *h = (WWWHead *)cldata;
  FrameHome(h->frame);
  return;
}

/*
 * Help
 */
void
Help(w, cldata, cbdata)
Widget w;
XtPointer cldata, cbdata;
{
  WWWHead *h = (WWWHead *)cldata;

  if (appr.helpURL != NULL)
  {
    URLParts *up;
    MemPool mp = MPCreate();
    if ((up = URLParse(mp, appr.helpURL)) != NULL) FrameLoad(h->frame, up);
    MPDestroy(mp);
  }

  return;
}

/*
 * sigiqh_handler
 */
static void
sigigh_handler()
{
  return;
}

/*
 * Dup
 */
void
Dup(w, cldata, cbdata)
Widget w;
XtPointer cldata, cbdata;
{
  Widget toplevel;
  WWWHead *h = (WWWHead *)cldata;

  toplevel = XtAppCreateShell(appr.argv0, "Chimera",
			      topLevelShellWidgetClass,
			      appr.dpy, NULL, 0);
  if (toplevel != 0) MakeHead(toplevel, h->surl);

  return;
}

/*
 * xErrorHandler
 */
static int
xErrorHandler(dpy, xe)
Display *dpy;
XErrorEvent *xe;
{
  fprintf (stderr, "X error\n");
  fflush(stderr);
  return(0);
}

/*
 * xtErrorHandler
 */
static void
xtErrorHandler(msg)
String msg;
{
  fprintf (stderr, "%s\n", msg);
  fflush(stderr);
  return;
}

/*
 * xtWarningHandler
 */
static void
xtWarningHandler(msg)
String msg;
{
  fprintf (stderr, "%s\n", msg);
  fflush(stderr);
  return;
}
