/*
 * setup.c
 *
 * Copyright (c) 1995, John Kilburg <john@cs.unlv.edu>
 * See the file COPYRIGHT for details.
 */
#include <stdio.h>

#include <sys/types.h>
#include <sys/stat.h>

/* Jim Rees fix */
#ifndef S_ISDIR
#define S_ISDIR(mode) (((mode) & S_IFMT) == S_IFDIR)
#endif

#include "WWW.h"

typedef struct _setup
{
  MemPool mp;
  IOTimeOut iot;
  Frame frame;
  URLParts *up;
} SetupInfo;

extern void InitChimeraBuiltins _ArgProto((WWWContext));

/*
 *
 * Setup Protocol
 *
 */

/*
 * PDestroy
 */
int
PDestroy(si)
SetupInfo *si;
{
  IODestroyTimeOut(si->iot);
  MPDestroy(si->mp);
  return(0);
}

/*
 * PTimeOut
 */
static void
PTimeOut(iot, si)
IOTimeOut iot;
SetupInfo *si;
{
  Document *d;

  d = DocumentCreate(true);
  d->mflist = MIMECreateField(d->mp, "content-type", "x-chimera/x-setup");
  if (si->up->filename != NULL) d->ds = alloc_string(si->up->filename);
  else d->ds = alloc_string("");
  d->data = d->ds;
  d->len = strlen(d->data);
 
  FrameInitData(si->frame, d);
  FrameEndData(si->frame, d);

  return;
}

/*
 * PSetup
 */
void *
PSetup(up, pup, closure, frame)
URLParts *up, *pup;
void *closure;
Frame frame;
{
  SetupInfo *si;
  MemPool mp;

  mp = MPCreate();
  si = (SetupInfo *)MPCGet(mp, sizeof(SetupInfo));
  si->iot = IOCreateTimeOut(frame, si);
  si->frame = frame;
  si->up = URLDup(mp, up);
  si->mp = mp;
  IOSetTimeOut(si->iot, 10, PTimeOut);
  return(si);
}

/*
 *
 * Setup Renderer
 *
 */

/*
 * RDestroy
 */
int
RDestroy(si)
SetupInfo *si;
{
  MPDestroy(si->mp);
  return(0);
}

/*
 * REnd
 */
int
REnd(si, d)
SetupInfo *si;
Document *d;
{
  struct stat s;
  char *rdir;
  char *tdir, *ctdir;
  char *cdir;
  char *sdir;
  URLParts *up;
  FILE *fp;
  char *croot = "~/.chimera";

  /*
   * Try to setup the chimera config directory.
   */
  rdir = FixPath(si->mp, croot);
  if (stat(rdir, &s) == -1) mkdir(rdir, 0700);

  tdir = (char *)MPGet(si->mp, strlen(croot) + strlen("/cache") + 1);
  strcpy(tdir, croot);
  strcat(tdir, "/cache");
  ctdir = tdir;

  cdir = FixPath(si->mp, tdir);
  if (stat(cdir, &s) == -1) mkdir(cdir, 0700);

  tdir = (char *)MPGet(si->mp, strlen(croot) + strlen("/resources") + 1);
  strcpy(tdir, croot);
  strcat(tdir, "/resources");

  sdir = FixPath(si->mp, tdir);
  if (stat(sdir, &s) == -1)
  {
    if ((fp = fopen(sdir, "w")) != NULL)
    {
      fprintf (fp, "! Cache configuration\n");
      fprintf (fp, "cache.directory: %s\n", ctdir);
      fprintf (fp, "! HTTP configuration\n");
      fprintf (fp, "http.user_agent: Chimera/2.0alpha\n");
      fclose(fp);
    }
  }
  
  WWWAddResourceFile(FrameToWWWContext(si->frame), sdir);

  /*
   * Pull in the rest of the modules now that everything is setup.
   */
  InitChimeraBuiltins(FrameToWWWContext(si->frame));

  /*
   * Load the real first document.
   */
  if ((up = URLParse(si->mp, si->up->filename)) != NULL)
  {
    FrameReplace(si->frame, up);
  }

  return(0);
}

/*
 * RSetup
 */
void *
RSetup(up, d, closure, frame)
URLParts *up;
Document *d;
void *closure;
Frame frame;
{
  SetupInfo *si;
  MemPool mp;

  /*
   * Get ready for setup information...
   */
  mp = MPCreate();
  si = (SetupInfo *)MPCGet(mp, sizeof(SetupInfo));
  si->frame = frame;
  si->mp = mp;
  si->up = URLDup(mp, up);

  return(si);
}

void
InitModule_Setup(wc)
WWWContext wc;
{
  static ProtoHooks ph;
  static RenderHooks rh;

  memset(&ph, 0, sizeof(ProtoHooks));
  ph.protocol = "chimera-setup";
  ph.init = PSetup;
  ph.destroy = PDestroy;
  WWWAddProtoHooks(wc, &ph);
  
  memset(&rh, 0, sizeof(RenderHooks));
  rh.content = "x-chimera/x-setup";
  rh.init = RSetup;
  rh.end = REnd;
  rh.destroy = RDestroy;
  WWWAddRenderHooks(wc, &rh);

  return;
}
