/*
 * widget.c
 *
 * Copyright (C) 1993, 1994, 1995, John D. Kilburg (john@cs.unlv.edu)
 * See the file COPYRIGHT for details.
 *
 * Note that most of this code was actually written by Jim.Rees@umich.edu.
 * The messed up parts were written by john.
 */
#include <stdio.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#include <X11/Xaw/Form.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/AsciiText.h>
#include "ScrollText.h"

#include "WWW.h"
#include "resources.h"

static struct ButtonTable
{
  char *name;
  Widget w;
  void (*cb)();
} ButtonTable[] =
{
  { "quit",     NULL, Quit },
  { "open",	NULL, Open },
  { "home",	NULL, Home },
  { "back",	NULL, Back },
  { "reload",   NULL, Reload },
  { "cancel",   NULL, Cancel },
  { "dup",	NULL, Dup },
  { "help",	NULL, Help },
  { NULL,       NULL, NULL },
};

static void
AddButtons(r, box, list)
WWWHead *r;
Widget box;
char *list;
{
  char name[256];
  struct ButtonTable *btp;

  while (sscanf(list, " %[^,]", name) == 1)
  {
    /*
     * Find the listed button and create its widget
     */
    for (btp = &ButtonTable[0]; btp->name != NULL; btp++)
    {
      if (!strcasecmp(btp->name, name))
      {
	btp->w = XtVaCreateManagedWidget(btp->name,
					 commandWidgetClass, box,
					 NULL);
	XtAddCallback(btp->w, XtNcallback, btp->cb, (XtPointer)r);
	break;
      }
    }

    /*
     * Fill in the entry in AppResources for the widget.
     */
    if (!strcasecmp(name, "open")) r->open = btp->w;
    else if (!strcasecmp(name, "back")) r->back = btp->w;
    else if (!strcasecmp(name, "reload")) r->reload = btp->w;
    else if (!strcasecmp(name, "cancel")) r->cancel = btp->w;
    else if (!strcasecmp(name, "quit")) r->quit = btp->w;
    else if (!strcasecmp(name, "home")) r->home = btp->w;
    else if (!strcasecmp(name, "help")) r->help = btp->w;
    else if (!strcasecmp(name, "dup")) r->dup = btp->w;

    /*
     * Skip to the next comma-delimited item in the list
     */
    while (*list && *list != ',')
      list++;
    if (*list == ',')
      list++;
  }

  return;
}

/*
 * CreateHead
 *
 * Setup chimera in a widget.
 */
WWWHead *
CreateHead(r, toplevel)
AppResources *r;
Widget toplevel;
{
  Widget paned, box, form;
  WWWHead *h;
  XFontStruct *f = r->messageFont;

  h = (WWWHead *)alloc_mem(sizeof(WWWHead));
  memset(h, 0, sizeof(WWWHead));
  h->toplevel = toplevel;
  h->group = alloc_string("default");
  h->next = r->hlist;
  r->hlist = h;

  /*
   * Main window pane
   */
  paned = XtCreateManagedWidget("paned",
                                panedWidgetClass, toplevel, 
                                NULL, 0);

  /*
   * Button pane(s)
   */
  if (r->button1Box && *r->button1Box)
  {
    box = XtCreateManagedWidget("box1", boxWidgetClass, paned, NULL, 0);
    AddButtons(h, box, r->button1Box);
  }

  if (r->button2Box && *r->button2Box)
  {
    box = XtCreateManagedWidget("box2", boxWidgetClass, paned, NULL, 0);
    AddButtons(h, box, r->button2Box);
  }

  /*
   * URL pane
   */
  form = XtCreateManagedWidget("urlform",
			       formWidgetClass, paned,
			       NULL, 0);
  XtCreateManagedWidget("urllabel",
			labelWidgetClass, form,
			NULL, 0);  
  h->url = XtVaCreateManagedWidget("url",
				   scrollingTextWidgetClass, form,
				   NULL);
  XtVaGetValues(h->url, XtNtextWidget, &(h->turl), NULL);
				   
  /*
   * Message pane.
   */
  h->message = XtVaCreateManagedWidget("message",
				       widgetClass, paned,
				       XtNheight, f->ascent + f->descent + 4,
				       XtNwidth, 1,
				       NULL);

  /*
   * WWW pane.
   */
  h->frame = WWWCreateToplevelFrame(r->wc, paned, "www",
				    URLCallback, MessageCallback, h);
  h->www = FrameWidget(h->frame);

  return(h);
}

/*
 * DestroyHead
 */
void
DestroyHead(r, h)
AppResources *r;
WWWHead *h;
{
  WWWHead *c, *t;

  for (t = NULL, c = r->hlist; c != NULL; c = c->next)
  {
    if (c == h)
    {
      if (t == NULL) r->hlist = c->next;
      else t->next = c->next;
      FrameDestroy(h->frame);
      XtDestroyWidget(h->toplevel);
      if (h->surl != NULL) free_mem(h->surl);
      free_mem(h);
      return;
    }
    t = c;
  }

  return;
}
