/*
 * WWW.c
 *
 * Copyright (c) 1995, John Kilburg <john@cs.unlv.edu>
 * See the file COPYRIGHT for details.
 */
#include <stdio.h>

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Scrollbar.h>

#include "WWWP.h"
#include "Geom.h"

static char defaultTranslations[] =
"<Btn1Up>:      select() \n\
 <Btn2Up>:      select() \n\
 <Expose>:      expose() \n\
 <Motion>:      motion() \n\
";

static XtActionsRec actionsList[] =
{
  { "expose", WWWExposeAction },
  { "select", WWWSelectAction },
  { "motion", WWWMotionAction },
};

#define offset(field) XtOffsetOf(WWWRec, www.field)
static XtResource resources[] =
{
  { XtNbackground, XtCBackground, XtRPixel, sizeof(Pixel),
	offset(background), XtRString, (XtPointer)"moccasin" },

  { XtNforeground, XtCForeground, XtRPixel, sizeof(Pixel),
	offset(foreground), XtRString, (XtPointer)"black" },
};
#undef offset

static void WWWInitialize(), WWWResize(), WWWRealize();
static Boolean WWWSetValues();
static XtGeometryResult WWWGeometryManager();
static XtGeometryResult WWWPreferredGeometry();

static void WWWGetValuesHook _ArgProto((Widget, ArgList, Cardinal *));

WWWClassRec wwwClassRec =
{
  { /* core_class fields */
    /* superclass         */    (WidgetClass) &compositeClassRec,
    /* class_name         */    "WWW",
    /* widget_size        */    sizeof(WWWRec),
    /* class_initialize   */    NULL,
    /* class_part_init    */    NULL,
    /* class_inited       */    FALSE,
    /* initialize         */    WWWInitialize,
    /* initialize_hook    */    NULL,
    /* realize            */    WWWRealize,
    /* actions            */    actionsList,
    /* num_actions        */    XtNumber(actionsList),
    /* resources          */    resources,
    /* num_resources      */    XtNumber(resources),
    /* xrm_class          */    NULLQUARK,
    /* compress_motion    */    FALSE,
    /* compress_exposure  */    FALSE,
    /* compress_enterleave*/    FALSE,
    /* visible_interest   */    FALSE,
    /* destroy            */    NULL,
    /* resize             */    WWWResize,
    /* expose             */    NULL,
    /* set_values         */    WWWSetValues,
    /* set_values_hook    */    NULL,
    /* set_values_almost  */    XtInheritSetValuesAlmost,
    /* get_values_hook    */    WWWGetValuesHook,
    /* accept_focus       */    NULL,
    /* version            */    XtVersion,
    /* callback_private   */    NULL,
    /* tm_table           */    NULL,
    /* query_geometry     */	WWWPreferredGeometry,
    /* display_accelerator*/	XtInheritDisplayAccelerator,
    /* extension          */	NULL
  },
  { /* composite_class fields */
    /* geometry_manager   */   WWWGeometryManager,
    /* change_managed     */   NULL,
    /* insert_child       */   XtInheritInsertChild,
    /* delete_child       */   XtInheritDeleteChild,
    /* extension          */   NULL
  },
  { /* www_class fields */
    NULL,
  }
};

WidgetClass wwwWidgetClass = (WidgetClass)&wwwClassRec;

/*
 * WWWGetValuesHook
 */
static void
WWWGetValuesHook(w, args, num_args)
Widget w;
ArgList args;
Cardinal * num_args;
{
  WWWWidget rw = (WWWWidget)w;
  int i;

  for (i = 0; i < *num_args; i++)
  {
    if (strcmp(args[i].name, XtNbackground) == 0)
    {
      *((Pixel *)args[i].value) = rw->www.background;
    }
    if (strcmp(args[i].name, XtNforeground) == 0)
    {
      *((Pixel *)args[i].value) = rw->www.foreground;
    }
  }

  return;
}

/*
 * WWWResize
 */
static void
WWWResize(w)
Widget w;
{
  return;
}

/*
 * WWWRealize
 */
static void
WWWRealize(w, valueMask, attributes)
Widget w;
XtValueMask *valueMask;
XSetWindowAttributes *attributes;
{
  (*wwwWidgetClass->core_class.superclass->core_class.realize)
      (w, valueMask, attributes);

  return;
}

static XtGeometryResult
WWWPreferredGeometry(w, con, reply)
Widget w;
XtWidgetGeometry *con, *reply;
{
  WWWWidget rw = (WWWWidget)w;

  XtResizeWidget(rw->www.clip, con->width > 0 ? con->width:100,
		 con->height > 0 ? con->height:100, 0);

  return(XtGeometryYes);
}

/*
 * WWWGeometryManager
 */
static XtGeometryResult
WWWGeometryManager(w, request, reply)
Widget w;
XtWidgetGeometry *request;
XtWidgetGeometry *reply;
{ 
  return(XtGeometryNo);
}

/*
 * WWWSetValues
 */
static Boolean
WWWSetValues(c, r, n)
Widget c, r, n;
{
  return(False);
}

/*
 * Scroll handling code snarfed from Viewport.c from Xaw.
 */
static void
SetBar(w, top, length, total)
Widget w;
Position top;
Dimension length, total;
{
  XawScrollbarSetThumb(w, (float)top/(float)total,
		       (float)length/(float)total);

  return;
}

static void
RedrawThumbs(rw)
WWWWidget rw;
{
  register Widget child = rw->www.child;
  register Widget clip = rw->www.clip;
  
  if (rw->www.horiz_bar != (Widget)NULL)
  {
    SetBar(rw->www.horiz_bar, -(child->core.x),
	   clip->core.width, child->core.width);
  }
  
  if (rw->www.vert_bar != (Widget)NULL)
  {
    SetBar(rw->www.vert_bar, -(child->core.y),
	   clip->core.height, child->core.height);
  }

  return;
}

static void
MoveChild(rw, x, y)
WWWWidget rw;
Position x, y;
{
  register Widget child = rw->www.child;
  register Widget clip = rw->www.clip;
  
  /* make sure we never move past right/bottom borders */
  if (-x + (int)clip->core.width > (int)child->core.width)
  {
    x = -(child->core.width - clip->core.width);
  }
  
  if (-y + (int)clip->core.height > (int)child->core.height)
  {
    y = -(child->core.height - clip->core.height);
  }
  
  /* make sure we never move past left/top borders */
  if (x >= 0) x = 0;
  if (y >= 0) y = 0;
  
  XtMoveWidget(child, x, y);
  
  RedrawThumbs(rw);

  return;
}

static void
ScrollUpDownProc(widget, closure, call_data)
Widget widget;
XtPointer closure;
XtPointer call_data;
{
  WWWWidget rw = (WWWWidget)XtParent(widget);
  register Widget child = rw->www.child;
  int pix = (int)call_data;
  Position x, y;
  
  if (child == NULL) return;  /* no child to scroll. */
  
  x = child->core.x - ((widget == rw->www.horiz_bar) ? pix : 0);
  y = child->core.y - ((widget == rw->www.vert_bar) ? pix : 0);
  MoveChild(rw, x, y);

  return;
}

static void
ThumbProc(widget, closure, percent)
Widget widget;
XtPointer closure;
float *percent;
{
  WWWWidget rw = (WWWWidget)XtParent(widget);
  register Widget child = rw->www.child;
  Position x, y;
  
  if (child == NULL) return;  /* no child to scroll. */
  
  if (widget == rw->www.horiz_bar)
  {
    x = -(int)(*percent * child->core.width);
  }
  else x = child->core.x;

  if (widget == rw->www.vert_bar)
  {
    y = -(int)(*percent * child->core.height);
  }
  else y = child->core.y;

  MoveChild(rw, x, y);

  return;
}

/* END SCROLL HANDLING CODE */

/*
 * WWWInitialize
 */
static void
WWWInitialize(r, n)
Widget r, n;
{
  WWWWidget new = (WWWWidget)n;

  /*
   * Avoid zero.
   */
  if (new->core.width < 5) new->core.width = 5;
  if (new->core.height < 5) new->core.height = 5;

  new->www.wc = NULL;

  /*
   * Create scrollbars, clip, and the child widget.  The child widget is
   * the one that we draw into.
   */
  new->www.vert_bar = XtVaCreateWidget("vert_bar",
				       scrollbarWidgetClass, n,
				       XtNorientation, XtorientVertical,
				       XtNmappedWhenManaged, False,
				       NULL);
  XtManageChild(new->www.vert_bar);

  XtAddCallback(new->www.vert_bar, XtNscrollProc,
		ScrollUpDownProc, (XtPointer)new);
  XtAddCallback(new->www.vert_bar, XtNjumpProc,
		ThumbProc, (XtPointer)new);
  
  new->www.horiz_bar = XtVaCreateWidget("horiz_bar",
					scrollbarWidgetClass, n,
					XtNorientation, XtorientHorizontal,
					XtNmappedWhenManaged, False,
					NULL);
  XtManageChild(new->www.horiz_bar);

  XtAddCallback(new->www.horiz_bar, XtNscrollProc,
		ScrollUpDownProc, (XtPointer)new);
  XtAddCallback(new->www.horiz_bar, XtNjumpProc,
		ThumbProc, (XtPointer)new);

  new->www.clip = XtVaCreateWidget("clip",
				   geomWidgetClass, n,
				   XtNwidth, new->core.width,
				   XtNheight, new->core.height,
				   XtNborderWidth, 0,
				   NULL);
  XtManageChild(new->www.clip);

  new->www.child = XtVaCreateWidget("child",
				    geomWidgetClass, new->www.clip,
				    XtNwidth, new->www.clip->core.width,
				    XtNheight, new->www.clip->core.height,
				    XtNborderWidth, 0,
				    NULL);
  XtManageChild(new->www.child);

  XtOverrideTranslations(new->www.child,
			 XtParseTranslationTable(defaultTranslations));

  return;
}
/*
 * FixDim
 */
static Dimension
FixDim(d)
unsigned int d;
{
  if (d > 0x7ffd) return(0x7ffd);
  return((Dimension)d);
}

/*
 * ConfigureWidgets
 */
static void
ConfigureWidgets(rw, width, height)
WWWWidget rw;
unsigned int width, height;
{
  Dimension child_width, child_height;
  Dimension clip_width, clip_height;
  Dimension vx, hy;
  Dimension vw, hh;
  float shown;

  if (!XtIsRealized((Widget)rw)) return;

  vw = rw->www.vert_bar->core.width;
  hh = rw->www.horiz_bar->core.height;

  child_width = FixDim(width);
  child_height = FixDim(height);

  clip_width = rw->core.width - vw;
  clip_height = rw->core.height - hh;

  if (child_width > clip_width) hy = hh;
  else
  {
    hy = 0;
    child_width = clip_width;
  }

  if (child_height > clip_height) vx = vw;
  else
  {
    vx = 0;
    child_height = clip_height;
  }

  if (child_height > clip_height)
  {
    XtConfigureWidget(rw->www.vert_bar,
		      0, hy,
		      vw, rw->core.height - hy, 0);
    shown = (float)clip_height / (float)child_height;
    XawScrollbarSetThumb(rw->www.vert_bar, (float)-1.0, shown);
    XtSetMappedWhenManaged(rw->www.vert_bar, True);
  }
  else XtSetMappedWhenManaged(rw->www.vert_bar, False);

  if (child_width > clip_width)
  {
    XtConfigureWidget(rw->www.horiz_bar,
		      vx, 0,
		      rw->core.width - vx, hh, 0);
    shown = (float)clip_width / (float)child_width;
    XawScrollbarSetThumb(rw->www.horiz_bar, (float)-1.0, shown);
    XtSetMappedWhenManaged(rw->www.horiz_bar, True);
  }
  else XtSetMappedWhenManaged(rw->www.horiz_bar, False);

  XtConfigureWidget(rw->www.clip,
		    vw, hh,
		    clip_width, clip_height, 0);

  XtResizeWidget(rw->www.child, child_width, child_height, 0);

  return;
}

/*
 * WWWDrawWidget
 */
Widget
WWWDrawWidget(w)
Widget w;
{
  WWWWidget rw = (WWWWidget)w;
  return(rw->www.child);
}

/*
 * WWWSetContext
 */
void
WWWSetContext(w, wc)
Widget w;
WWWContext wc;
{
  WWWWidget rw = (WWWWidget)w;
  rw->www.wc = wc;
  return;
}

/*
 * WWWGetSize
 */
int
WWWGetSize(w, with_scroll, width, height)
Widget w;
bool with_scroll;
unsigned int *width, *height;
{
  WWWWidget rw = (WWWWidget)w;

  if (!rw->core.mapped_when_managed) return(-1);

  if (!with_scroll)
  {
    *width = (unsigned int)(rw->core.width);
    *height = (unsigned int)(rw->core.height);
  }
  else
  {
    *width = (unsigned int)(rw->core.width - rw->www.vert_bar->core.width);
    *height = (unsigned int)(rw->core.height - rw->www.horiz_bar->core.height);
  }

  return(0);
}

/*
 * WWWSetSize
 */
int
WWWSetSize(w, with_scroll, width, height)
Widget w;
bool with_scroll;
unsigned int width, height;
{
  WWWWidget rw = (WWWWidget)w;

  if (with_scroll)
  {
    XtResizeWidget(w,
		   FixDim(width) + rw->www.vert_bar->core.width,
		   FixDim(height) + rw->www.horiz_bar->core.height, 0);
  }
  else
  {
    XtResizeWidget(w, FixDim(width), FixDim(height), 0);
  }

  return(0);
}

/*
 * WWWSetDrawSize
 */
int
WWWSetDrawSize(w, with_scroll, width, height)
Widget w;
bool with_scroll;
unsigned int width, height;
{
  WWWWidget rw = (WWWWidget)w;

  if (!with_scroll)
  {
    XtSetMappedWhenManaged(rw->www.vert_bar, False);
    XtSetMappedWhenManaged(rw->www.horiz_bar, False);
    XtConfigureWidget(rw->www.clip, 0, 0, rw->core.width, rw->core.height, 0);
    XtConfigureWidget(rw->www.child, 0, 0, rw->core.width, rw->core.height, 0);
  }
  else
  {
    ConfigureWidgets(rw, width, height);
  }

  return(0);
}

/*
 * WWWDrawReset
 */
int
WWWDrawReset(w)
Widget w;
{
  WWWWidget rw = (WWWWidget)w;

  XClearWindow(XtDisplay(rw->www.child), XtWindow(rw->www.child));
  XtMoveWidget(rw->www.child, 0, 0);
  return(0);
}
