/*
 * WWWP.h
 *
 * Copyright (c) 1995, John D. Kilburg <john@cs.unlv.edu>
 * See the file COPYRIGHT for details.
 */
#ifndef __WWWP_H__
#define __WWWP_H__

#include "WWW.h"

typedef struct _WWWClassRec  *WWWWidgetClass;
typedef struct _WWWRec	     *WWWWidget;

extern WidgetClass           wwwWidgetClass;

typedef struct _realm
{
  char *name;
  char *type;
  char *protocol;
  char *hostname;
  int port;
  char *username;
  char *password;
  int used;
  struct _realm *next;
} Realm;

typedef struct _stack
{
  MemPool mp;                         /* memory pool for this stack */
  Document *d;                        /* address of document information */
  RenderHooks *crh;                   /* cache render hooks */
  void *crc;                          /* cache render context */
  int action;                         /* type of action that triggered this */
  bool norender;                      /* could not get renderer */
  ProtoHooks *ph;                     /* proto hooks */
  void *pc;                           /* proto context */
  RenderHooks *rh;                    /* render hooks */
  void *rc;                           /* render context */
  URLParts *up;                       /* URL that triggered this mess */
  URLParts *pup;                      /* proxy URL */
  float vert_shown, vert_top;         /* vertical scrollbar position */
  float horiz_shown, horiz_top;       /* horizontal scrollbar position */
  struct _stack *next;
  struct _Frame *frame;               /* parent frame */
} Stack;

struct _Frame
{
  /* X private */
  Widget      www;                    /* WWW widget */

  /* other private */
  MemPool     mp;                     /* memory pool */
  char        *name;                  /* name of frame */
  bool        destroyed;              /* FrameDestroy() called ? */
  Stack       *slist;                 /* URL stack */
  Stack       *sp;                    /* URL pending */
  void        (*size_callback)();     /* called when the size changes */
  void        *size_closure;
  WWWMessageProc message_callback;    /* called when a message is sent */
  void        *message_closure;
  WWWURLProc  url_callback;           /* called when the URL changes */
  void        *url_closure;
  bool        size_set;
  bool        use_override;           /* check override hooks ? */
  RenderHooks orh;                    /* override render hooks */
  void        *orc;                   /* override render context */
  struct _WWWContext *wc;
  struct _Frame *parent;              /* parent frame */
  struct _Frame *kids;                /* list of child frames */
  struct _Frame *knext;               /* next kid */
  struct _Frame *fnext;               /* next frame */
  struct _Frame *toplevel;            /* toplevel ancestor */
};

struct _WWWContext
{
  MemPool     mp;                     /* memory pool */
  XrmDatabase db;                     /* message database */
  ProtoHooks  *phlist;                /* proto hooks list */
  RenderHooks *rhlist;                /* render hooks list */
  ConvHooks   *chlist;                /* converter hooks list */
  Realm       *rlist;                 /* cached auth realms list */
  struct _Frame *flist;               /* frame list */
  WWWResolveProc mod_resolve;         /* module resolver */
  void        *mod_closure;           /* module resolver closure */
  int         iocnt;                  /* IO connection count */
};

typedef struct
{
  XtPointer extension;
} WWWClassPart;

typedef struct _WWWClassRec
{
  CoreClassPart	        core_class;
  CompositeClassPart	composite_class;
  WWWClassPart	        www_class;
} WWWClassRec;

extern WWWClassRec wwwClassRec;

typedef struct _WWWPart
{
  /* Public */
  Pixel       foreground;
  Pixel       background;

  /* X Private */
  Widget      vert_bar, horiz_bar;  /* scroll bars */
  Widget      clip;                 /* clips the child */
  Widget      child;                /* the drawing window */
  Widget      menu;                 /* menu widget */

  /* other private */
  struct _WWWContext *wc;
} WWWPart;

typedef struct _WWWRec
{
  CorePart		core;
  CompositePart     	composite;
  WWWPart		www;
} WWWRec;

/*
 * WWW.c
 */
Widget WWWDrawWidget _ArgProto((Widget));
void WWWSetContext _ArgProto((Widget, WWWContext));
int WWWSetSize _ArgProto((Widget, bool, unsigned int, unsigned int));
int WWWSetDrawSize _ArgProto((Widget, bool, unsigned int, unsigned int));
int WWWGetSize _ArgProto((Widget, bool, unsigned int *, unsigned int *));
int WWWDrawReset _ArgProto((Widget));

/*
 * action.c
 */
void WWWExposeAction();
void WWWSelectAction();
void WWWMotionAction();

/*
 * proxy.c
 */
URLParts *GetProxy _ArgProto((MemPool, URLParts *));

#endif
