/*
 * context.c
 *
 * Copyright (c) 1995, John Kilburg <john@cs.unlv.edu>
 * See the file COPYRIGHT for details.
 */

#include <stdio.h>

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include "WWWP.h"

WWWContext
WWWCreateContext(mod_resolve, mod_closure)
bool (*mod_resolve)();
void *mod_closure;
{
  struct _WWWContext *wc;
  MemPool mp;

  mp = MPCreate();
  wc = (struct _WWWContext *)MPCGet(mp, sizeof(struct _WWWContext));
  wc->mp = mp;
  wc->mod_resolve = mod_resolve;
  wc->mod_closure = mod_closure;
  return(wc);
}

void
WWWAddResourceFile(wc, file)
struct _WWWContext *wc;
char *file;
{
  char *dbfile;
  MemPool mp;

  mp = MPCreate();
  dbfile = FixPath(mp, file);

  if (wc->db == NULL) wc->db = XrmGetFileDatabase(dbfile);
  else XrmCombineFileDatabase(dbfile, &(wc->db), True);

  MPDestroy(mp);

  return;
}

void
WWWDestroyContext(wc)
struct _WWWContext *wc;
{
  RenderHooks *rh;
  ConvHooks *ch;
  ProtoHooks *ph;
  struct _Frame *f, *t;

  for (f = wc->flist; f != NULL; f = f->fnext)
  {
    FrameDestroy(f);
  }

  for (rh = wc->rhlist; rh != NULL; rh = rh->next)
  {
    if (rh->class_destroy != NULL) (rh->class_destroy)(rh->class_context);
  }
  for (ch = wc->chlist; ch != NULL; ch = ch->next)
  {
    if (ch->class_destroy != NULL) (ch->class_destroy)(ch->class_context);
  }
  for (ph = wc->phlist; ph != NULL; ph = ph->next)
  {
    if (ph->class_destroy != NULL) (ph->class_destroy)(ph->class_context);
  }
  if (wc->db != NULL) XrmDestroyDatabase(wc->db);

  for (f = wc->flist; f != NULL; )
  {
    t = f;
    f = f->fnext;
    MPDestroy(t->mp);
  }

  MPDestroy(wc->mp);

  return;
}

/*
 * WWWAddProtoHooks
 */
int
WWWAddProtoHooks(wc, phooks)
struct _WWWContext *wc;
ProtoHooks *phooks;
{
  phooks->next = wc->phlist;
  wc->phlist = phooks;
  return(0);
}

/*
 * WWWAddRenderHooks
 */
int
WWWAddRenderHooks(wc, rhooks)
struct _WWWContext *wc;
RenderHooks *rhooks;
{
  rhooks->next = wc->rhlist;
  wc->rhlist = rhooks;
  return(0);
}

/*
 * WWWGetResource
 */
char *
WWWGetResource(wc, name)
struct _WWWContext *wc;
char *name;
{
  char *type;
  XrmValue v;

  if (wc->db != NULL)
  {
    if (!XrmGetResource(wc->db, name, XtNstring, &type, &v)) return(NULL);
    return(v.addr);
  }

  return(NULL);
}

/*
 * WWWAddConvHooks
 */
int
WWWAddConvHooks(wc, chooks)
struct _WWWContext *wc;
ConvHooks *chooks;
{
  chooks->next = wc->chlist;
  wc->chlist = chooks;
  return(0);
}

/*
 * WWWCreateToplevelFrame
 */
Frame
WWWCreateToplevelFrame(wc, parent, name, url_cb, message_cb, closure)
struct _WWWContext *wc;
void *parent;
char *name;
void (*url_cb)();
void (*message_cb)();
void *closure;
{
  struct _Frame *frame;

  frame = FrameCreate(wc, NULL);
  frame->www = XtVaCreateManagedWidget(name,
				       wwwWidgetClass, (Widget)parent,
				       NULL);
  WWWSetContext(frame->www, wc);
  frame->size_set = true;
  frame->url_callback = url_cb;
  frame->url_closure = closure;
  frame->message_callback = message_cb;
  frame->message_closure = closure;
  frame->toplevel = frame;

  return(frame);
}

/*
 * FrameToWWWContext
 */
WWWContext
FrameToWWWContext(frame)
struct _Frame *frame;
{
  return(frame->wc);
}

/*
 * FrameFindNamed
 */
Frame
WWWFindNamedFrame(wc, name)
struct _WWWContext *wc;
char *name;
{
  struct _Frame *f;
  size_t nlen = strlen(name);

  for (f = wc->flist; f != NULL; f = f->fnext)
  {
    if (f->name != NULL && strlen(f->name) == nlen &&
	strcmp(f->name, name) == 0 && !f->destroyed)
    {
      return(f);
    }
  }

  return(NULL);
}

