/*
 * gui.c
 *
 * Copyright (c) 1995, John D. Kilburg <john@cs.unlv.edu>
 * See the file COPYRIGHT for details.
 */
#include <stdio.h>

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>

#include <X11/Xaw/Scrollbar.h>

#include "WWWP.h"

int
FrameMessage(frame, message)
struct _Frame *frame;
char *message;
{
  struct _Frame *tl = frame->toplevel;

  if (tl != NULL && tl->message_callback != NULL)
  {
    (tl->message_callback)(tl,
			   tl->message_closure,
			   message);
  }

  return(0);
}

/*
 * Mmmmm, long name.
 */
void
FrameGetOnScreenDimensions(frame, x, y, width, height)
struct _Frame *frame;
int *x, *y;
unsigned int *width, *height;
{
  WWWWidget rw = (WWWWidget)frame->www;

  *x = -(int)rw->www.child->core.x;
  *y = -(int)rw->www.child->core.y;
  *width = (unsigned int)rw->www.child->core.width;
  *height = (unsigned int)rw->www.clip->core.height;
     
  return;
}

int
FrameMap(frame, x, y)
struct _Frame *frame;
int x, y;
{
  XtMoveWidget(frame->www, (Position)x, (Position)y);
  XtSetMappedWhenManaged(frame->www, True);
  return(0);
}

int
FrameGetSize(frame, with_scroll, width, height)
struct _Frame *frame;
bool with_scroll;
unsigned int *width, *height;
{
  if (!frame->size_set) return(-1);
  WWWGetSize(frame->www, with_scroll, width, height);
  return(0);
}

int
FrameReturnSize(frame, with_scroll, width, height)
struct _Frame *frame;
bool with_scroll;
unsigned int width, height;
{
  WWWSetSize(frame->www, with_scroll, width, height);
  frame->size_set = True;
  if (frame->size_callback != NULL)
  {
    (frame->size_callback)(frame->size_closure, 0, width, height);
  }
  return(0);
}

int
FrameSetSize(frame, with_scroll, width, height)
struct _Frame *frame;
bool with_scroll;
unsigned int width, height;
{
  WWWSetDrawSize(frame->www, with_scroll, width, height);
  return(0);
}

Frame
FrameCreateUnsized(parent, callback, closure)
struct _Frame *parent;
void (*callback)();
void *closure;
{
  struct _Frame *frame;

  if (parent == NULL || parent->www == NULL) return(NULL);

  frame = FrameCreate(parent->wc, parent);
  frame->www = XtVaCreateManagedWidget("www",
				       wwwWidgetClass,
				       WWWDrawWidget(parent->www),
				       XtNmappedWhenManaged, False,
				       XtNborderWidth, 0,
				       NULL);
  WWWSetContext(frame->www, parent->wc);
  frame->size_callback = callback;
  frame->size_closure = closure;
  frame->size_set = false;

  return(frame);
}

Frame
FrameCreateSized(parent, width, height)
struct _Frame *parent;
unsigned int width, height;
{
  struct _Frame *frame;

  if (parent == NULL || parent->www == NULL) return(NULL);

  frame = FrameCreate(parent->wc, parent);
  frame->www = XtVaCreateManagedWidget("www_child",
				       wwwWidgetClass,
				       WWWDrawWidget(parent->www),
				       XtNmappedWhenManaged, False,
				       XtNborderWidth, 0,
				       XtNwidth, (Dimension)width,
				       XtNheight, (Dimension)height,
				       NULL);
  WWWSetContext(frame->www, parent->wc);
  frame->size_set = true;

  return(frame);
}

int
FrameGetNamedColor(frame, name, pixel)
struct _Frame *frame;
char *name;
Pixel *pixel;
{
  XColor sxc, exc;
  Display *dpy = XtDisplay(frame->www);

  XAllocNamedColor(dpy, DefaultColormap(dpy, DefaultScreen(dpy)),
                   name, &sxc, &exc);

  *pixel = sxc.pixel;

  return(0);
}

void *
FrameDrawWidget(frame)
struct _Frame *frame;
{
  return(WWWDrawWidget(frame->www));
}

void *
FrameWidget(frame)
struct _Frame *frame;
{
  return(frame->www);
}

char *
FrameGetResource(frame, name)
struct _Frame *frame;
char *name;
{
  return(WWWGetResource(frame->wc, name));
}

int
FrameGetAuth(frame, func, closure)
struct _Frame *frame;
void (*func)();
void *closure;
{
  return(0);
}
