/*
 * mime.c
 *
 * Copyright (C) 1994, 1995, John Kilburg (john@cs.unlv.edu)
 * See the file COPYRIGHT for details.
 */
#include <stdio.h>
#include <ctype.h>

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include "WWW.h"

typedef struct _mimetype
{
  char *content;
  char *ext;
  struct _mimetype *next;
} MIMEType;

static MIMEType def_mtlist[] =
{
  { "text/html", "html" },
  { "text/html", "htm" },
  { "text/plain", "txt" },
  { "image/gif", "gif" },
  { "image/png", "png" },
  { "image/xbm", "xbm" },
  { "image/x-xpimap", "xpm" },
  { "image/x-portable-anymap", "pnm" },
  { "image/x-portable-bitmap", "pbm" },
  { "image/x-portable-graymap", "pgm" },
  { "image/x-portable-pixmap", "ppm" },
  { "image/jpeg", "jpg" },
  { "image/jpeg", "jpeg" },
  { "image/tiff", "tiff" },
  { "image/tiff", "tif" },
  { "image/x-fits", "fit" },
  { "image/x-fits", "fits" },
  { "image/x-fits", "fts" },
  { "audio/basic", "au" },
  { "audio/basic", "snd" },
  { "text/x-compress-html", "html.Z" },
  { "text/x-gzip-html", "html.gz" },
  { "application/postscript", "ps" },
  { "application/x-dvi", "dvi" },
  { "application/x-gzip", "gz" },
  { "application/x-compress", "Z" },
  { "application/x-tar", "tar" },
  { "video/mpeg", "mpeg" },
  { "video/mpeg", "mpg" },
  { NULL, NULL },
};

static MIMEType *mtlist = NULL;

/*
 * MIMEReadTypes
 */
void
MIMEReadTypeFiles(filelist)
char *filelist;
{
  FILE *fp;
  char *f;
  char *cp;
  char buffer[256];
  char content[256];
  char exts[256];
  MIMEType *t = NULL, *c;
  char *filename, *e;
  MemPool mp;

  mp = MPCreate();

  f = filelist;
  while ((filename = mystrtok(f, ':', &f)) != NULL)
  {
    filename = FixPath(mp, filename);
    if (filename == NULL) continue;

    fp = fopen(filename, "r");
    if (fp == NULL) continue;
    
    while (fgets(buffer, sizeof(buffer), fp))
    {
      if (buffer[0] == '#' || buffer[0] == '\n') continue;
      
      if (sscanf(buffer, "%s %[^\n]", content, exts) == 2)
      {
	cp = exts;
	while ((e = mystrtok(cp, ' ', &cp)) != NULL)
	{
	  c = (MIMEType *)alloc_mem(sizeof(MIMEType));
	  memset(c, 0, sizeof(MIMEType));
	  c->content = alloc_string(content);
	  c->ext = alloc_mem(strlen(e) + 2);
	  strcpy(c->ext, ".");
	  strcat(c->ext, e);
	  c->next = NULL;
	  if (t != NULL) t->next = c;
	  else mtlist = c;
	  t = c;
	}
      }
    }
    
    fclose(fp);
  }

  MPDestroy(mp);

  return;
}

/*
 * MIMEExt2Content
 */
char *
MIMEExt2Content(ext)
char *ext;
{
  int elen, flen;
  MIMEType *c;
  int i;

  flen = strlen(ext);
  for (c = mtlist; c; c = c->next)
  {
    elen = strlen(c->ext);
    if (elen > 0)
    {
      if (elen <= flen &&
	  strncasecmp(ext + flen - elen, c->ext, elen) == 0)
      {
	return(c->content);
      }
    }
  }

  for (i = 0; def_mtlist[i].content != NULL; i++)
  {
    elen = strlen(def_mtlist[i].ext);
    if (elen > 0)
    {
      if (elen <= flen &&
	  strncasecmp(ext + flen - elen, def_mtlist[i].ext, elen) == 0)
      {
	return(def_mtlist[i].content);
      }
    }
  }

  return(NULL);
}

static MIMECap def_mclist[] =
{
  { "application/postscript", "ghostview %s" },
  { "application/x-dvi", "xdvi %s" },
  { "image/x-fits", "saoimage %s" },
  { "image/*", "xv %s" },
  { "video/mpeg", "mpeg_play -loop %s" },
  { "audio/basic", "playaudio %s" },
  { NULL, NULL },
};

static MIMECap *mclist = NULL;

/*
 * MIMEReadCapFiles
 *
 * Read a list of mailcap files.
 */
void
MIMEReadCapFiles(filelist)
char *filelist;
{
  FILE *fp;
  char *f;
  char buffer[256];
  char ctype[256];
  char command[256];
  char junk[256];
  MIMECap *c, *t = NULL;
  char *filename;
  MemPool mp;

  mp = MPCreate();

  f = filelist;
  while ((filename = mystrtok(f, ':', &f)) != NULL)
  {
    filename = FixPath(mp, filename);
    if (filename == NULL) continue;
   
    fp = fopen(filename, "r");
    if (fp == NULL) continue;
    
    while (fgets(buffer, sizeof(buffer), fp))
    {
      if (buffer[0] == '#' || buffer[0] == '\n') continue;
      
      if (sscanf(buffer, "%[^;]%[;]%[^;\n]", ctype, junk, command) == 3)
      {
	c = (MIMECap *)alloc_mem(sizeof(MIMECap));
	memset(c, 0, sizeof(MIMECap));
	c->ctype = alloc_string(ctype);
	c->command = alloc_string(command);
	c->next = NULL;
	if (t != NULL) t->next = c;
	else mclist = c;
	t = c;
      }
    }

    fclose(fp);
  }

  MPDestroy(mp);

  return;
}

/*
 * MatchTypes
 *
 * Hell, you'd almost think that John would want to use some sort of
 * regular expression thing here instead of this shit.
 */
static int
MatchTypes(t1, t2)
char *t1, *t2;
{
  int len, dlen;

  len = strlen(t1);
  if (t1[0] != '*')
  {
    if (t1[len - 1] == '*') len--;
    if (strncasecmp(t1, t2, len) == 0) return(0);
  }
  else
  {
    dlen = strlen(t2);
    if (dlen >= len)
    {
      if (strncasecmp(t1 + 1, t2 + (dlen - len), len - 1) == 0) return(0);
    }
  }

  return(-1);
}

/*
 * FindViewer
 */
MIMECap *
MIMEFindViewer(content)
char *content;
{
  MIMECap *c;
  int i;

  for (c = mclist; c != NULL; c = c->next)
  {
    if (MatchTypes(c->ctype, content) == 0) return(c);
  }

  for (i = 0; def_mclist[i].ctype != NULL; i++)
  {
    if (MatchTypes(def_mclist[i].ctype, content) == 0) return(&def_mclist[i]);
  }

  return(NULL);
}

/*
 * ParseExpiresDate
 *
 * This functions sucks.
 */
/*
time_t
ParseExpiresDate(s)
char *s;
{
#ifdef HAVE_MKTIME
  int i;
  time_t expires;
  struct tm tm;
  char buffer[BUFSIZ];
  char month[4], zone[4];
  static char *ms[] =
  { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct",
    "Nov", "Dec" };

  memset(&tm, 0, sizeof(tm));
  if (sscanf(s, "%s %d %s %d %d:%d:%d %s",
	     buffer, &tm.tm_mday, month, &tm.tm_year,
	     &tm.tm_hour, &tm.tm_min, &tm.tm_sec, zone) == 8)
  {
    if (tm.tm_year > 1900) tm.tm_year -= 1900;
    for (i = 0; i < sizeof(ms) / sizeof(ms[0]); i++)
    {
      if (strcmp(month, ms[i]) == 0)
      {
	expires = mktime(&tm);
	break;
      }
    }
  }
  
  return(expires);
#else
  return(0);
#endif
}
*/

/*
 * MIMECreateField
 */
MIMEField *
MIMECreateField(mp, name, value)
MemPool mp;
char *name;
char *value;
{
  MIMEField *mf;

  mf = (MIMEField *)MPGet(mp, sizeof(MIMEField));
  mf->name = MPStrDup(mp, name);
  mf->value = MPStrDup(mp, value);
  mf->next = NULL;
  return(mf);
}

static MIMEField *MIMEParseField _ArgProto((MemPool, char *, size_t));

/*
 * MIMEParseField
 *
 * Expects the entire field.
 *
 * This is a disaster when it comes to parsing MIME stuff.
 */
static MIMEField *
MIMEParseField(mp, line, len)
MemPool mp;
char *line;
size_t len;
{
  char *cp, *start;
  size_t i;
  char *name;
  char *value;
  MIMEField *mf;

  for (cp = line, i = 0; *cp != ':' && i < len; i++, cp++)
      ;
  if (i == len) return(NULL);

  name = (char *)MPGet(mp, cp - line + 1);
  strncpy(name, line, cp - line);
  name[cp - line] = '\0';
  
  for (cp++; isspace(*cp) && i <  len; cp++, i++)
      ;
  if (i == len) return(NULL);

  for (start = cp;
       *cp != ';' && *cp != '\r' && *cp != '\n' && i < len;
       cp++, i++)
      ;

  value = (char *)MPGet(mp, cp - start + 1);
  strncpy(value, start, cp - start);
  value[cp - start] = '\0';

  mf = MIMECreateField(mp, name, value);

  return(mf);
}

/*
 * MIMEParseHeader
 *
 * Expects the entire header.
 *
 * This is a disaster when it comes to parsing MIME stuff.
 */
MIMEField *
MIMEParseHeader(mp, line, len)
MemPool mp;
char *line;
size_t len;
{
  size_t i;
  char *cp, *start;
  MIMEField *mflist, *c;

  mflist = NULL;
  for (i = 0, cp = line, start = line; i < len; i++, cp++)
  {
    if (*cp == '\r') continue;
    else if (*cp == '\n')
    {
      if ((c = MIMEParseField(mp, start, cp - start)) != NULL)
      {
	c->next = mflist;
	mflist = c;
      }
      start = cp + 1;
    }
  }

  return(mflist);
}

/*
 * MIMEGetField
 */
char *
MIMEGetField(mflist, name)
MIMEField *mflist;
char *name;
{
  MIMEField *m;

  for (m = mflist; m != NULL; m = m->next)
  {
    if (strcasecmp(m->name, name) == 0) break;
  }

  if (m == NULL) return(NULL);

  return(m->value);
}

/*
 * MIMEFindData
 *
 * Returns the first byte of the data in a MIME deal.
 * Returns -1 if no data.
 */
int
MIMEFindData(data, len)
char *data;
size_t len;
{
  size_t i;

  for (i = 0; i < len; i++)
  {
    if (data[i] == '\n' &&
	(data[i + 1] == '\n' || (data[i + 1] == '\r' && data[i + 2] == '\n')))
    {
      if (data[i + 1] == '\n') return(i + 2);
      return(i + 3);
    }
  }

  return(-1);
}
