/*
 * url.c
 *
 * Copyright (C) 1993, 1994, 1995, John D. Kilburg <john@cs.unlv.edu>
 * See the file COPYRIGHT for details.
 */
#include <stdio.h>
#include <ctype.h>

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include "WWW.h"
#include "md5.h"

#ifndef NullString
#define NullString(s)	(s == NULL || *s == '\0')
#endif

static URLParts *URLCreate _ArgProto((MemPool));

/*
 * URLcmp
 *
 * Return 0 if equal.
 */
int
URLcmp(u1, u2)
URLParts *u1, *u2;
{
  if (strcasecmp(u1->protocol, u2->protocol) == 0 &&
      strcasecmp(u1->hostname, u2->hostname) == 0 &&
      u1->port == u2->port &&
      strcasecmp(u1->filename, u2->filename) == 0)
  {
    return(0);
  }
  return(-1);
}

/*
 * URLEscape
 *
 * Puts escape codes in URLs.  NOT complete.
 */
char *
URLEscape(mp, url, s2p)
MemPool mp;
char *url;
bool s2p;
{
  char *cp;
  char *n, *s;
  static char *hex = "0123456789ABCDEF";

  /*
   * use a bit of memory so i don't have to mess around here
   */
  s = n = (char *)MPGet(mp, strlen(url) * 3 + 2);

  for (cp = url; *cp; cp++, n++)
  {
    if (*cp == ' ' && s2p)
    {
      *n = '+';
    }
    else if (*cp == '+' && s2p)
    {
      *n = '%';
      n++;
      *n = hex[*cp / 16];
      n++;
      *n = hex[*cp % 16];
    }
    else if (isalnum(*cp) || strchr("$-_.'(),+!*", *cp))
    {
      *n = *cp;
    }
    else
    {
      *n = '%';
      n++;
      *n = hex[*cp / 16];
      n++;
      *n = hex[*cp % 16];
    }
  }

  *n = '\0';

  return(s);
}

/*
 * UnescapeURL
 *
 * Converts the escape codes (%xx) into actual characters.  NOT complete.
 * Could do everthing in place I guess.
 */
char *
URLUnescape(mp, url)
MemPool mp;
char *url;
{
  char *cp, *n, *s;
  char hex[3];

  s = n = (char *)MPGet(mp, strlen(url) + 2);
  for (cp = url; *cp; cp++, n++)
  {
    if (*cp == '%')
    {
      cp++;
      if (*cp == '%')
      {
	*n = *cp;
      }
      else
      {
	hex[0] = *cp;
	cp++;
	hex[1] = *cp;
	hex[2] = '\0';
	*n = (char)strtol(hex, NULL, 16);
      }
    }
    else
    {
      *n = *cp;
    }
  }

  *n = '\0';

  return(s);
}

/*
 * URLMakeString
 */
char *
URLMakeString(mp, up, addanchor)
MemPool mp;
URLParts *up;
int addanchor;
{
  size_t len;
  char *u;
  char *delim;
  char *delim2;
  char *filename;
  char *hostname;
  char *protocol;
  char *delim3;
  char *anchor;

  if (up->protocol != NULL && strncmp(up->protocol, "urn", 3) == 0)
  {
    u = (char *)MPGet(mp, strlen(up->filename) + 5);
    strcpy(u, "urn:");
    strcat(u, up->filename);
    return(u);
  }

  if (NullString(up->protocol)) protocol = "file";
  else protocol = up->protocol;
  
  if (NullString(up->hostname))
  {
    delim = "";
    hostname = "";
  }
  else
  {
    delim = "//";
    hostname = up->hostname;
  }

  if (NullString(up->filename)) filename = "/";
  else filename = up->filename;

  delim2 = "";

  if (up->anchor != NULL && addanchor != 0)
  {
    anchor = up->anchor;
    delim3 = "#";
  }
  else
  {
    anchor = "";
    delim3 = "";
  }

  len = strlen(protocol) + strlen(hostname) + strlen(filename) +
        strlen(delim) + strlen(anchor) + 11;
  u = (char *)MPGet(mp, len + 1);
  if (up->port == 0)
  {
    sprintf (u, "%s:%s%s%s%s%s%s", protocol, delim, hostname, delim2,
	     filename, delim3, anchor);
  }
  else
  {
    sprintf (u, "%s:%s%s:%d%s%s%s%s", protocol, delim, hostname, up->port,
           delim2, filename, delim3, anchor);
  }

  return(u);
}

/*
 * URLCreate
 *
 * Allocate URLParts and initialize to NULLs
 */
static URLParts *
URLCreate(mp)
MemPool mp;
{
  URLParts *up;

  up = (URLParts *)MPCGet(mp, sizeof(URLParts));
  up->reload = false;

  return(up);
}

/*
 * URLResolve
 */
URLParts *
URLResolve(mp, up1, up2)
MemPool mp;
URLParts *up1, *up2;
{
  URLParts *r;
  char *filename;

  /*
   * Special case for URNs.  Sucks.
   */
  if (up1->protocol != NULL && strncmp(up1->protocol, "urn", 3) == 0)
  {
    return(URLDup(mp, up1));
  }

  r = URLCreate(mp);

  /*
   * Deal with the protocol.  If it has a protocol then use it, otherwise
   * use the parent's protocol.  If the parent doesn't have a protocol for
   * some spooky reason then use "file".
   */
  if (up1->protocol == NULL)
  {
    if (up2->protocol != NULL) r->protocol = MPStrDup(mp, up2->protocol);
    else r->protocol = MPStrDup(mp, "file");
  }
  else r->protocol = MPStrDup(mp, up1->protocol);

  /*
   * Deal with the hostname.  If it has a hostname then use it, otherwise
   * if the protocol used by the parent is the same then use the parent's
   * hostname.
   *
   * The port goes along with the hostname.
   */
  if (up1->hostname == NULL)
  {
    /*
     * r is used here because up1->protocol can be NULL but r->protocol
     * will always have the right thing.
     */
    if (up2->hostname != NULL && up2->protocol != NULL &&
        strcasecmp(r->protocol, up2->protocol) == 0)
    {
      r->hostname = MPStrDup(mp, up2->hostname);
      r->port = up2->port;
    }
  }
  else
  {
    r->hostname = MPStrDup(mp, up1->hostname);
    r->port = up1->port;
  }

  if (up1->filename == NULL)
  {
    r->filename = MPStrDup(mp, "/");
  }
  if (up2->protocol == NULL || strcasecmp(r->protocol, up2->protocol) != 0)
  {
    r->filename = MPStrDup(mp, up1->filename);
  }
  else if (up1->filename[0] == '/' || up1->filename[0] == '~')
  {
    r->filename = MPGet(mp, strlen(up1->filename) + 2);
    if (up1->filename[0] != '/')
    {
      r->filename[0] = '/';
      strcpy(r->filename + 1, up1->filename);
    }
    else
    {
      strcpy(r->filename, up1->filename);
    }
  }
  else
  {
    /*
     * This includes an attempt to deal with '..' and '.'
     */
    size_t i;
    size_t len, len2;
    size_t start;
    size_t backcount = 0;

    backcount = 0;
    start = 0;
    len = strlen(up1->filename);
    for (i = 0; i < len; i++)
    {
      if (strncmp(up1->filename + i, "../", 3) == 0)
      {
	start = i + 3;
	i += 2;
	backcount++;
      }
      else if (strncmp(up1->filename + i, "./", 2) == 0)
      {
	start = i + 2;
	i += 1;
      }
    }

    filename = MPStrDup(mp, up2->filename);
    for (i = strlen(filename) - 1; i >= 0; i--)
    {
      if (filename[i] == '/')
      {
	if (backcount-- == 0)
	{
	  filename[i + 1] = '\0';
	  break;
	}
      }
    }

    if (i == -1)
    {
      r->filename = MPStrDup(mp, up1->filename + start);
    }
    else
    {
      len2 = strlen(filename);
      r->filename = MPGet(mp, len2 + strlen(up1->filename + start) + 1);
      strcpy(r->filename, filename);
      strcat(r->filename, up1->filename + start);
    }
  }

  /*
   * Copy misc. fields.
   */
  r->method = MPStrDup(mp, up1->method);
  r->auth_realm = MPStrDup(mp, up1->auth_realm);
  r->auth_type = MPStrDup(mp, up1->auth_type);
  r->username = MPStrDup(mp, up1->username);
  r->password = MPStrDup(mp, up1->password);
  r->anchor = MPStrDup(mp, up1->anchor);

  if (up1->input_data != NULL)
  {
    r->input_data = MPGet(mp, up1->input_data_len);
    memcpy(r->input_data, up1->input_data, up1->input_data_len);
    r->input_data_len = up1->input_data_len;
    r->input_data_type = MPStrDup(mp, up1->input_data_type);
  }
  else r->input_data_len = 0;

  /*
   * Need the parent's reload status.
   */
  r->reload = up2->reload;

  return(r);
}

URLParts *
URLDup(mp, up)
MemPool mp;
URLParts *up;
{
  URLParts *dp;

  dp = URLCreate(mp);
  dp->method = MPStrDup(mp, up->method);
  dp->protocol = MPStrDup(mp, up->protocol);
  dp->hostname = MPStrDup(mp, up->hostname);
  dp->port = up->port;

  dp->filename = up->filename != NULL ?
      MPStrDup(mp, up->filename):MPStrDup(mp, "/");

  dp->anchor = MPStrDup(mp, up->anchor);


  dp->auth_realm = MPStrDup(mp, up->auth_realm);
  dp->auth_type = MPStrDup(mp, up->auth_type);
  dp->username = MPStrDup(mp, up->username);
  dp->password = MPStrDup(mp, up->password);

  if (up->input_data != NULL)
  {
    dp->input_data = MPGet(mp, up->input_data_len);
    memcpy(dp->input_data, up->input_data, up->input_data_len);
    dp->input_data_len = up->input_data_len;
    dp->input_data_type = MPStrDup(mp, dp->input_data_type);
  }
  else dp->input_data_len = 0;

  dp->reload = up->reload;

  return(dp);
}

/*
 * URLParse
 *
 * Turns a URL into a URLParts structure
 *
 * The good stuff was written by Rob May <robert.may@rd.eng.bbc.co.uk>
 * and heavily mangled/modified by john to suite his own weird style.
 */
URLParts *
URLParse(mp, url)
MemPool mp;
char *url;
{
  URLParts *up;
  char *start;
  char *colon, *slash, *fslash;
  char *pound; /* link pound (#) sign */
  char *at; /* username/password @ */
  char *ucolon; /* username colon */
  char *pcolon; /* port number colon */

  up = URLCreate(mp);

  /* skip leading white-space (if any)*/
  for (start = url; isspace(*start); start++)
      ;

  /* Lousy hack for URNs */
  if (strncasecmp(url, "urn:", 4) == 0)
  {
    up->protocol = MPStrDup(mp, "urn");
    up->filename = MPStrDup(mp, url + 4);
    return(up);
  }

  colon = strchr(start, ':');
  slash = strchr(start, '/');

  /*
   * Check to see if there is a protocol.
   */
  if (colon != NULL && (slash == NULL || colon < slash))
  {
    up->protocol = MPGet(mp, colon - start + 1);
    strncpy(up->protocol, start, colon - start);
    up->protocol[colon - start] = '\0';
  }

  /*
   * If there is a slash then sort out the hostname and filename.
   * If there is no slash then there is no hostname but there is a
   * filename.
   */
  if (slash != NULL)
  {
    /*
     * Check for leading //. If its there then there is a host string.
     */
    if ((*(slash + 1) == '/') && ((colon == NULL && slash == start) ||
	(colon != NULL && slash == colon + 1)))
    {
      /*
       * Check for filename at end of host string.
       */
      slash += 2;
      if ((fslash = strchr(slash, '/')) != NULL)
      {
	up->hostname = MPGet(mp, fslash - slash + 1);
	strncpy(up->hostname, slash, fslash - slash);
	up->hostname[fslash - slash] = '\0';
	up->filename = MPStrDup(mp, fslash);
      }
      else
      { /* there is no filename */
	up->hostname = MPStrDup(mp, slash);
      }
    }
    else
    {
      /*
       * the rest is a filename because there is no // or it appears
       * after other characters
       */
      if (colon != NULL && colon < slash) 
      {
	up->filename = MPStrDup(mp, colon + 1);
      }
      else up->filename = MPStrDup(mp, start);
    }
  }
  else
  {
    /*
     * No slashes at all so the rest must be a filename.
     */
    if (colon == NULL) up->filename = MPStrDup(mp, start);
    else up->filename = MPStrDup(mp, colon + 1);
  }

  /*
   * If there is a host string then divide it into
   * username:password@hostname:port as needed.
   */
  if (up->hostname != NULL)
  {
    /*
     * Look for username:password.
     */
    if ((at = strchr(up->hostname, '@')) != NULL)
    {
      char *mumble;

      up->username = MPGet(mp, at - up->hostname + 1);
      strncpy(up->username, up->hostname, at - up->hostname);
      up->username[at - up->hostname] = '\0';

      mumble = MPStrDup(mp, at + 1);
      up->hostname = mumble;

      if ((ucolon = strchr(up->username, ':')) != NULL)
      {
	up->password = MPStrDup(mp, ucolon + 1);
	*ucolon = '\0';
      }
    }

    /*
     * Grab the port.
     */
    if ((pcolon = strchr(up->hostname, ':')) != NULL)
    {
      up->port = atoi(pcolon + 1);
      *pcolon = '\0';
    }
  }

  /*
   * Check the filename for a '#foo' string.
   */
  if (up->filename != NULL)
  {
    if ((pound = strchr(up->filename, '#')) != NULL)
    {
      *pound = '\0';
      up->anchor = MPStrDup(mp, pound + 1);
    }
  }
  else up->filename = MPStrDup(mp, "/");

  return(up);
}


/*
 * URLIsAbsolute
 */
bool
URLIsAbsolute(up)
URLParts *up;
{
  if (up->protocol == NULL) return(false);
  return(true);
}

/*
 * URLHash
 */
char *
URLHash(mp, up)
MemPool mp;
URLParts *up;
{
  struct MD5Context context;
  unsigned char digest[16];
  unsigned char *ep, *dp;
  char *cp;
  char *ustr;
  size_t ulen;
  char *hex;

  /* prevent memory problems...ports should never be this big anyways */
  if (up->port > 99999999) return(NULL);

  ulen = 10;    /* for the port number.  trouble! */
  if (up->protocol != NULL) ulen += strlen(up->protocol);
  if (up->hostname != NULL) ulen += strlen(up->hostname);
  if (up->filename != NULL) ulen += strlen(up->filename);
/*
  if (up->method != NULL) ulen += strlen(up->method);
  if (up->input_data != NULL) ulen += up->input_data_len;
*/

  ustr = (char *)MPGet(mp, sizeof(char) * (ulen + 1));
  sprintf (ustr, "%d", up->port);
  if (up->protocol != NULL) strcat(ustr, up->protocol);
  if (up->hostname != NULL) strcat(ustr, up->hostname);
  if (up->filename != NULL) strcat(ustr, up->filename);
/*
  if (up->method != NULL) strcat(ustr, up->method);
  if (up->input_data != NULL)
  {
    memcpy(ustr + strlen(ustr), up->input_data, up->input_data_len);
  }
*/

  MD5Init(&context);
  MD5Update(&context, ustr, (unsigned int)strlen(ustr));
  MD5Final(digest, &context);

  hex = (char *)MPGet(mp, 33);
  cp = hex;
  for (dp = digest, ep = digest + (sizeof(digest) / sizeof(digest[0]));
           dp < ep; cp += 2)
  {
    (void)sprintf(cp, "%02x", *dp++ & 0xff);
  }
  *cp = '\0';

  return(hex);
}
